/*--

-- File name 	: 	UART_RXFIFO.v

--             :

-- Title       :  UART RX FIFO (512 x 8 bits)

--             :

-- Purpose     :  buffering receiver data

--					:

--             :

--             :

--             :

-- Created On  : 03/20/02

--             :

-- Comments    : Design done by Insight Design Services

--             :

--					:

--					:

--             :

-- Conventions :

--					: 

--             : 

--   				:

--   				:

--   				:

-- Limitations : For use in Xilinx SPARTAN2 FPGA

--   				:

-- Developers  : ERIC ZHANG

-- ----------------------------------------------------------------------

-- Revision History :

-- ----------------------------------------------------------------------

--   Version No:| Author    	| Changes Made: 						| Mod. Date:

--     v1.0     | EZ  			| First Written						| 

-- ------------------------------------------------------------------------*/

`timescale 1 ns/1 ps





module UART_RXFIFO (

   RESET,

   CLK,

   RX_PUSH,

   RX_PUSH_DATA,

   

   RX_POP,

   RX_EMPTY,

   RX_POP_DATA,

   RX_OF,

   RX_COUNT

   );   

   

input          RESET;

input          CLK;

input          RX_PUSH;

input  [7:0]   RX_PUSH_DATA;

   

input          RX_POP;

output         RX_EMPTY;

output [7:0]   RX_POP_DATA;

output         RX_OF;

output [8:0]   RX_COUNT;



reg            RX_OF;

reg    [8:0]   RX_COUNT;



//------------------------------------------------------------------------------

//  Variable declarations

//------------------------------------------------------------------------------

reg  [8:0]  addr_wr;

wire [8:0]  next_addr_wr;

reg  [8:0]  addr_rd;

wire [8:0]  next_addr_rd;



reg      FULL, EMPTY;



wire        valid_push;

wire        valid_pop;



wire [7:0] pZero8;

wire        vcc;

wire        gnd;



assign pZero8 = 'h0;

assign vcc     = 1;

assign gnd     = 0;



//------------------------------------------------------------------------------

//  Begin Module

//------------------------------------------------------------------------------

assign RX_EMPTY = EMPTY;



assign valid_push = RX_PUSH & ~FULL;

assign valid_pop  = RX_POP & ~EMPTY;

assign next_addr_wr = addr_wr + 1;

assign next_addr_rd = addr_rd + 1;

//-------------------------------------------------

//---------------- FIFO status flag  --------------

//-------------------------------------------------

always @ (posedge RESET or posedge CLK)

   if (RESET) FULL <= 0;

   else if (RX_PUSH && (next_addr_wr == addr_rd)) FULL <= 1;

   else if (addr_wr != addr_rd) FULL <= 0;

      

      

always @ (posedge RESET or posedge CLK)

   if (RESET) EMPTY <= 1;

   else if (RX_POP && (next_addr_rd == addr_wr)) EMPTY <= 1;

   else if (addr_wr != addr_rd) EMPTY <= 0;

      

always @ (posedge RESET or posedge CLK)

   if (RESET)  RX_OF <= 0;

   else      RX_OF <= FULL & RX_PUSH;

      

always @ (posedge RESET or posedge CLK)

   if (RESET)  RX_COUNT <= 'b0;

   else      RX_COUNT <= addr_wr - addr_rd;

      

//-------------------------------------------------

//------------ FIFO write/read address -----------

//-------------------------------------------------

always @ (posedge RESET or posedge CLK)

   if (RESET) addr_wr <= 'b0;

   else if (valid_push) addr_wr <= next_addr_wr;

      

// FIFO read address 

always @ (posedge RESET or posedge CLK)

   if (RESET) addr_rd <= 'b0;

   else if (valid_pop) addr_rd <= next_addr_rd;

      

//-------------------------------------------------

//--------- BLOCK RAM instances  --------------

//-------------------------------------------------



RAMB4_S8_S8  iUART_RXFIFO_BRAM

       (.DIA(RX_PUSH_DATA),

        .DIB(pZero8),

        .ENA(vcc),

        .ENB(vcc),

        .WEA(valid_push),

        .WEB(gnd),

        .RSTA(RESET),

        .RSTB(RESET),

        .CLKA(CLK),

        .CLKB(CLK),

        .ADDRA(addr_wr),

        .ADDRB(addr_rd),

        .DOA(),

        .DOB(RX_POP_DATA)

       ); 





endmodule