-------------------------------------------------------------------------

--

-- File name   : bar0.vhd

--             :  

-- Title       : 

--             :  

-- Functional  : 

-- Description : bar0 of pci design

--             : 

-- Created On  : 8/7/00   

--             :

-- Testbed     :

-- Requirements: <testing requirements -- explain relation to spec.>

--             :

-- Coding      :  

-- Standard    : Active low signals are identified with '_n' or '_N'.

--             : Only one entity-architecture pair per file.

--             :

-- Synthesis   :

-- Standard    : All outputs registered when possible.

--             :

-- Synthesis   :

-- Warnings    : <NO warnings without written explanation>

--             :

-- Device      :

-- Utilization : <est. clb count, tbufs, dffs, etc.>

--             :

-- Limitations : For use in Xilinx FPGA

--             : 

-- Developers  : Tom Curran

--             : Memec Design

--             : 30 Nagog Park

--             : Acton, MA  01720

--             : Ph 978.266.9193

--             : Fx 978.266.9194

-- ----------------------------------------------------------------------

-- Revision History :

-- ----------------------------------------------------------------------

--   Version   | Author          | Changes Made:      | Mod. Date:

--      1.0    | tom curran      | First write        | 8/7/00      

-- ----------------------------------------------------------------------



library ieee;

use ieee.std_logic_1164.all;

use ieee.std_logic_unsigned.all;



entity bar0 is

port(

   clk : in std_logic;

   reset : in std_logic;

   s_data : in std_logic;

   s_data_vld : in std_logic;

   s_wrdn : in std_logic;

   base_hit : in std_logic;

   bar_hit : out std_logic;

   bar_intr_n : out std_logic;

   bar_keepout : out std_logic;

   bar_ready : out std_logic;

   bar_sabort : out std_logic;

   bar_term : out std_logic;

   adio : inout std_logic_vector(31 downto 0);

   led_test : out std_logic;

   tensdigit : out std_logic_vector(6 downto 0);

   onesdigit : out std_logic_vector(6 downto 0)

   );

end bar0;



architecture rtl of bar0 is



--components

   component displayrom

   port (

      clk : in std_logic;

      reset : in std_logic;

      romaddr : in std_logic_vector(6 downto 0);

      romdata : out std_logic_vector(13 downto 0)

      );

   end component;



   component phase_control

   port (

      rst            : in std_logic;            -- Global Reset

      clk            : in std_logic;            -- PCLK

      busy           : in std_logic;

      base_hit       : in std_logic;

      bar_wr         : in std_logic;

      bar_rd         : in std_logic;

      s_data         : in std_logic;

      w_ready        : out std_logic;

      r_ready        : out std_logic;

      w_term         : out std_logic;

      r_term         : out std_logic

      );

   end component;



   component bar_decode

   port (

      clk : in std_logic;

      reset : in std_logic;

      s_data : in std_logic;

      s_wrdn : in std_logic;

      decoded_hit : in std_logic;

      bar_hit : out std_logic;

      bar_rd : out std_logic;

      bar_wr : out std_logic

      );

   end component ;



   --signals

   signal bar_oe : std_logic;

   signal bar_rd : std_logic;

   signal bar_wr : std_logic;

   signal bar_we : std_logic;

   signal w_term : std_logic;

   signal r_term : std_logic;

   signal w_ready : std_logic;

   signal r_ready : std_logic;

   signal i_led_test : std_logic;

   signal increment : std_logic;

   signal decrement : std_logic;

   signal romaddr : std_logic_vector(6 downto 0) ;

   signal romdata : std_logic_vector(13 downto 0) ;

      

begin

   

   tensdigit <= romdata(13 downto 7) ;

   onesdigit <= romdata(6 downto 0) ;

   

   led_test <= i_led_test;

   

   adio(0) <= i_led_test when (bar_oe = '1') else 'Z';



   bar_we <= bar_wr and s_data_vld;



   bar_oe <= bar_rd and s_data;



   bar_ready <= w_ready or r_ready;

   bar_term <= w_term or r_term;

   

   increment <= bar_we and adio(0);

   decrement <= bar_we and not adio(0);



   static: process (clk,reset)

   begin

      if reset='1' then --asynchronous reset active high

         bar_sabort <= '1';

         bar_keepout <= '1';

         bar_intr_n <= '0';

      elsif (clk'event and clk='1') then  --clk rising edge

         bar_sabort <= '0';

         bar_keepout <= '0';

         bar_intr_n <= '1';

      end if;

   end process;

   

--component instances

   data_phase_control: phase_control   

   port map(

      rst => reset,       

      clk => clk,       

      busy => bar_we,

      base_hit => base_hit,

      bar_wr => bar_wr,    

      bar_rd => bar_rd,    

      s_data => s_data,    

      w_ready => w_ready,   

      r_ready => r_ready,   

      w_term  => w_term,   

      r_term  => r_term

      );



   bar_hit_0 : bar_decode

   port map(

      reset => reset,

      clk => clk,

      s_data => s_data,

      s_wrdn => s_wrdn,

      decoded_hit => base_hit,

      bar_hit => bar_hit,

      bar_rd => bar_rd,

      bar_wr => bar_wr

      );



   digitrom : displayrom

   port map(

      clk => clk,

      reset => reset,

      romaddr => romaddr,

      romdata => romdata

      );



   addrcnt : process(clk,reset)

   begin

      if (reset = '1') then

         romaddr <= (others => '0') ; 

      elsif (clk'event and clk = '1') then

         if (increment = '1') then

            romaddr <= romaddr + '1' ;

         elsif (decrement = '1') then

            romaddr <= romaddr - '1' ;

         end if;

      end if;

   end process;



   foo_data: process (clk,reset)

   begin

      if reset='1' then --asynchronous reset active high

         i_led_test <= '0';

      elsif (clk'event and clk='1') then  --clk rising edge

         if (bar_we ='1') then

            i_led_test <= adio(0);

         end if;

      end if;

   end process;

   

end rtl;

