/*--

-- File name 	: 	BLOCK_RAM_MODULE.v

--             :

-- Title       :  Block Ram Module

--             :

-- Purpose     :  This is the register interface for the sample SDRAM design

--					:

--             :

--             :

--             :

-- Created On  : October 22, 2001

--             :

-- Comments    : Design done by Insight Design Services

--             :

--					:

--					:

--             :

-- Conventions :

--					: Signal names ending in _int indicate the internal version of an output signal

--             : Tab size: 4

--   				:

--   				:

--   				:

-- Limitations : For use in Xilinx Virtex E FPGA

--   				:

-- Developers  : Duncan Allen

-- ----------------------------------------------------------------------

-- Revision History :

-- ----------------------------------------------------------------------

--   Version No:| Author    	| Changes Made: 						| Mod. Date:

--     v1.0     | da  			| First Written						| 10/22/01

-- ------------------------------------------------------------------------*/



module REGISTER_MODULE (

            CLK,

            RESET,



            BLOCK_RAM_WRITE_HIT,

            BLOCK_RAM_READ_HIT,

            S_DATA,

            ADDRESS,



            BLOCK_RAM_WRITE_DATA,

            BLOCK_RAM_READ_ENABLE,

            BLCOK_RAM_READ_DATA,



            BLOCK_RAM_READY,

            BLOCK_RAM_TERM,

            BLOCK_RAM_ABORT

            );





   input CLK;

   input RESET;

   input REGISTER_WRITE_HIT;

   input REGISTER_READ_HIT;

   input S_DATA;

   input [3:0]ADDRESS;



   input [31:0] 	REGISTER_WRITE_DATA;

   output   		REGISTER_READ_ENABLE;

   output [31:0]  REGISTER_READ_DATA;



   output REGISTER_READY;

   output REGISTER_TERM;

   output REGISTER_ABORT;





   wire register_read;

   wire register_write;



   reg [31:0] register_0;

   reg [31:0] register_1;

   reg [31:0] register_2;

   reg [31:0] register_3;

   reg [31:0] register_4;

   reg [31:0] register_5;

   reg [31:0] register_6;

   reg [31:0] register_7;

   reg [31:0] register_8;

   reg [31:0] register_9;

   reg [31:0] register_10;

   reg [31:0] register_11;

   reg [31:0] register_12;

   reg [31:0] register_13;

   reg [31:0] register_14;

   reg [31:0] register_15;



   reg [31:0] read_data;



   REGISTER_HIT_MACHINE iREG_MACHINE (

            .REGISTER_READ_HIT	(REGISTER_READ_HIT),

				.REGISTER_WRITE_HIT	(REGISTER_WRITE_HIT),

				.S_DATA					(S_DATA),

				.CLK						(CLK),

				.RESET					(RESET),

				.REGISTER_READ			(register_read),

				.REGISTER_WRITE		(register_write),

				.REGISTER_CLEAR		(),

				.REGISTER_ABORT		(REGISTER_ABORT),

				.REGISTER_TERM			(REGISTER_TERM),

				.REGISTER_READY      (REGISTER_READY)

				);





   // Process to load register_0

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_0 = 32'h12345678;

      else if (register_write && (ADDRESS ==4'b0000) && S_DATA)

         register_0 = REGISTER_WRITE_DATA;

      end



   // Process to load register_1

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_1 = 32'h9ABCDEF0;

      else if (register_write && (ADDRESS == 4'b0001) && S_DATA)

         register_1 = REGISTER_WRITE_DATA;

      end



      // Process to load register_2

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_2 = 32'h22222222;

      else if (register_write && (ADDRESS == 4'b010) && S_DATA)

         register_2 = REGISTER_WRITE_DATA;

      end



      // Process to load register_3

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_3 = 32'h33333333;

      else if (register_write && (ADDRESS == 4'b0011) && S_DATA)

         register_3 = REGISTER_WRITE_DATA;

      end



      // Process to load register_4

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_4 = 32'h44444444;

      else if (register_write && (ADDRESS == 4'b0100) && S_DATA)

         register_4 = REGISTER_WRITE_DATA;

      end



      // Process to load register_5

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_5 = 32'h55555555;

      else if (register_write && (ADDRESS == 4'b0101) && S_DATA)

         register_5 = REGISTER_WRITE_DATA;

      end



      // Process to load register_6

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_6 = 32'h66666666;

      else if (register_write && (ADDRESS == 4'b0110) && S_DATA)

         register_6 = REGISTER_WRITE_DATA;

      end



      // Process to load register_7

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_7 = 32'h77777777;

      else if (register_write && (ADDRESS == 4'b0111) && S_DATA)

         register_7 = REGISTER_WRITE_DATA;

      end



      // Process to load register_8

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_8 = 32'h88888888;

      else if (register_write && (ADDRESS == 4'b1000) && S_DATA)

         register_8 = REGISTER_WRITE_DATA;

      end



      // Process to load register_9

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_9 = 32'h99999999;

      else if (register_write && (ADDRESS == 4'b1001) && S_DATA)

         register_9 = REGISTER_WRITE_DATA;

      end



      // Process to load register_10

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_10 = 32'hAAAAAAAA;

      else if (register_write && (ADDRESS == 4'b1010) && S_DATA)

         register_10 = REGISTER_WRITE_DATA;

      end



      // Process to load register_11

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_11 = 32'hBBBBBBBB;

      else if (register_write && (ADDRESS == 4'b1011) && S_DATA)

         register_11 = REGISTER_WRITE_DATA;

      end



      // Process to load register_12

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_12 = 32'hCCCCCCCC;

      else if (register_write && (ADDRESS == 4'b1100) && S_DATA)

         register_12 = REGISTER_WRITE_DATA;

      end



      // Process to load register_13

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_13 = 32'hDDDDDDDD;

      else if (register_write && (ADDRESS == 4'b1101) && S_DATA)

         register_13 = REGISTER_WRITE_DATA;

      end



      // Process to load register_14

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_14 = 32'hEEEEEEEE;

      else if (register_write && (ADDRESS == 4'b1110) && S_DATA)

         register_14 = REGISTER_WRITE_DATA;

      end



      // Process to load register_15

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         register_15 = 32'hFFFFFFFF;

      else if (register_write && (ADDRESS == 4'b1111) && S_DATA)

         register_15 = REGISTER_WRITE_DATA;

      end







always @ (ADDRESS or register_0 or register_1 or register_2 or register_3 or register_4 or register_5 or register_6 or register_7 or

			register_8 or register_9 or register_10 or register_11 or register_12 or register_13 or register_14 or register_15)

   begin

   case (ADDRESS)

      4'b0000 : read_data = register_0;

      4'b0001 : read_data = register_1;

      4'b0010 : read_data = register_2;

      4'b0011 : read_data = register_3;

      4'b0100 : read_data = register_4;

      4'b0101 : read_data = register_5;

      4'b0110 : read_data = register_6;

      4'b0111 : read_data = register_7;

      4'b1000 : read_data = register_8;

      4'b1001 : read_data = register_9;

      4'b1010 : read_data = register_10;

      4'b1011 : read_data = register_11;

      4'b1100 : read_data = register_12;

      4'b1101 : read_data = register_13;

      4'b1110 : read_data = register_14;

      4'b1111 : read_data = register_15;

      default : read_data = register_0;

   endcase

end



assign REGISTER_READ_DATA = read_data;

assign REGISTER_READ_ENABLE = register_read;



endmodule