/*----------------------------------------------------------------------------------------------------

--

-- File name	 : sdram_top.V

--					 :

-- Title			 : SDRAM Top

--					 :

-- Purpose		 : The top level containg the userapp and the PCI core

--					 :

-- IncludeFiles :

--					 :

-- Created On	 : August 23, 2001

--					 :

-- Comments		 :

--					 :

--					 :

--					 :

--					 :  For the Spartan 2 Reference Board

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

-- Conventions  : - Port names are upper case.

--					 : - Internal signals and variables are lower case.

--					 : - Active low signals are identified with '_n' or '_N'.

--					 : - constants begin with lower case 'c', rest upper case

--					 : - states begin with lower case 's', rest upper case

--					 : - aliases begin with lower case 'a', rest upper case

--					 :

--					 : Tab setting = 3

--					 : Right Margin = 100

--					 :

-- Limitations  : For use in Xilinx FPGA

--					 :

-- Developers	 : Duncan Allen

----------------------------------------------------------------------------------------------------

-- Revision History :

----------------------------------------------------------------------------------------------------

--	  Version No 	| Author	 	| Changes Made:							| Mod. Date:

--		v1.0			| da			| First write								| 23/8/01

----------------------------------------------------------------------------------------------------*/



// This is a simple synchronous FIFO.

// It assumes that each burst of data will be 8 units long and that

// each burst will be emptied before a new one is loaded





`timescale 1 ns/1 ps



module sdram_top (

                AD,             // PCI ports -- do not modify names!

                CBE,

                PAR,

                FRAME_N,

                TRDY_N,

                IRDY_N,

                STOP_N,

                DEVSEL_N,

                IDSEL,

                INTR_A,

                PERR_N,

                SERR_N,

                REQ_N,

                GNT_N,

                RST_N,

                PCLK,

				   // The SDRAM Interface

    				SDRAM_A,

					SDRAM_BA,

					SDRAM_CLK,

					SDRAM_CKE,

					SDRAM_CS_L,

					SDRAM_RAS_L,

					SDRAM_CAS_L,

					SDRAM_WE_L,

					SDRAM_DQM,

					SDRAM_DQ,

               // The seven segment LED signals

               DISPLAY1A,

               DISPLAY1B,

               DISPLAY1C,

               DISPLAY1D,

               DISPLAY1E,

               DISPLAY1F,

               DISPLAY1G,

               DISPLAY2A,

               DISPLAY2B,

               DISPLAY2C,

               DISPLAY2D,

               DISPLAY2E,

               DISPLAY2F,

               DISPLAY2G,  

               // DIP SWITCH INPUTS

               DIP,

               // IO PORTS

               SIG_IN,

               SIG_OUT,

               // OTHERS

            	LED_USER,

               PUSH_USER1,

               FPGA_RESETN,

               RXD,

               TXD

                );

//------------------------------------------------------------------------------

// Synplicity directives

//------------------------------------------------------------------------------

// synthesis syn_edif_bit_format = "%u<%i>"

// synthesis syn_edif_scalar_format = "%u"

// synthesis syn_noclockbuf = 1



  inout  [31:0] AD;             // PCI ports -- do not modify names!

  inout   [3:0] CBE;

  inout         PAR;

  inout         FRAME_N;

  inout         TRDY_N;

  inout         IRDY_N;

  inout         STOP_N;

  inout         DEVSEL_N;

  input         IDSEL;

  output        INTR_A;

  inout         PERR_N;

  inout         SERR_N;

  output        REQ_N;

  input         GNT_N;

  input         RST_N;

  input         PCLK;



 	output [10:0]	SDRAM_A; 	// The SDRAM address

	output [1:0]   SDRAM_BA;		// The SDRAM bank select

	output 			SDRAM_CLK;		// The SDRAM clock

	output			SDRAM_CKE;		// The SDRAM clock enable

	output 			SDRAM_CS_L;	// The SDRAM chip select

	output 			SDRAM_RAS_L;	// The SDRAM row address strobe

	output         SDRAM_CAS_L;	// The SDRAM column address strobe\

	output 			SDRAM_WE_L;	// The SDRAM write enable

	output [3:0]	SDRAM_DQM;		// The SDRAM data masks

	inout [31:0]  	SDRAM_DQ;    // The SDRAM data



   output			DISPLAY1A;

   output         DISPLAY1B;

   output         DISPLAY1C;

   output         DISPLAY1D;

   output         DISPLAY1E;

   output         DISPLAY1F;

   output         DISPLAY1G;

   output         DISPLAY2A;

   output         DISPLAY2B;

   output         DISPLAY2C;

   output         DISPLAY2D;

   output         DISPLAY2E;

   output         DISPLAY2F;

   output     		DISPLAY2G;  

   

   input  [7:0]   DIP;

   input [53:0]   SIG_IN;

   output [53:0]  SIG_OUT;

   

   input          PUSH_USER1;

   input          FPGA_RESETN;

   input          TXD;

   output         RXD;

   output         LED_USER;

  // Internal buses -- do not modify names!



  wire  [255:0] CFG;

  wire   [31:0] ADDR;

  wire   [31:0] ADIO;

  wire    [7:0] BASE_HIT;

  wire    [3:0] S_CBE;

  wire   [15:0] PCI_CMD;

  wire    [3:0] M_CBE;

  wire   [39:0] CSR;

  wire   [31:0] SUB_DATA;





  wire          CLK;

  wire          RST;

  wire          reset;



  // Signals used with the DLL

  wire          CLK_FB;

  wire          CLK_FB_O;

  wire          SDRAM_CLK_OUT;

  wire          sout;

  wire          sin;



  assign RXD = sout;

  assign sin = TXD;

  assign reset = RST | ~FPGA_RESETN;





  // Instantiation of PCI Interface -- do not modify names!



  pcim_lc PCI_CORE (

                .AD_IO ( AD ),

                .CBE_IO ( CBE ),

                .PAR_IO ( PAR ),

                .FRAME_IO ( FRAME_N ),

                .TRDY_IO ( TRDY_N ),

                .IRDY_IO ( IRDY_N ),

                .STOP_IO ( STOP_N ),

                .DEVSEL_IO ( DEVSEL_N ),

                .IDSEL_I ( IDSEL ),

                .INTA_O ( INTR_A ),

                .PERR_IO ( PERR_N ),

                .SERR_IO ( SERR_N ),

                .REQ_O ( REQ_N ),

                .GNT_I ( GNT_N ),

                .RST_I ( RST_N ),

                .PCLK ( PCLK ),

                .FRAMEQ_N ( FRAMEQ_N ),

                .TRDYQ_N ( TRDYQ_N ),

                .IRDYQ_N ( IRDYQ_N ),

                .STOPQ_N ( STOPQ_N ),

                .DEVSELQ_N ( DEVSELQ_N ),

                .ADDR ( ADDR ),

                .ADIO ( ADIO ),

                .CFG_VLD ( CFG_VLD ),

                .CFG_HIT ( CFG_HIT ),

                .C_TERM ( C_TERM ),

                .C_READY ( C_READY ),

                .ADDR_VLD ( ADDR_VLD ),

                .BASE_HIT ( BASE_HIT ),

                .S_TERM ( S_TERM ),

                .S_READY ( S_READY ),

                .S_ABORT ( S_ABORT ),

                .S_WRDN ( S_WRDN ),

                .S_SRC_EN ( S_SRC_EN ),

                .S_DATA_VLD ( S_DATA_VLD ),

                .S_CBE ( S_CBE ),

                .PCI_CMD ( PCI_CMD ),

                .REQUEST ( REQUEST ),

                .REQUESTHOLD ( REQUESTHOLD ),

                .COMPLETE ( COMPLETE ),

                .M_WRDN ( M_WRDN ),

                .M_READY ( M_READY ),

                .M_SRC_EN ( M_SRC_EN ),

                .M_DATA_VLD ( M_DATA_VLD ),

                .M_CBE ( M_CBE ),

                .TIME_OUT ( TIME_OUT ),

                .CFG_SELF ( CFG_SELF ),

                .M_DATA ( M_DATA ),

                .DR_BUS ( DR_BUS ),

                .I_IDLE ( I_IDLE ),

                .M_ADDR_N ( M_ADDR_N ),

                .IDLE ( IDLE ),

                .B_BUSY ( B_BUSY ),

                .S_DATA ( S_DATA ),

                .BACKOFF ( BACKOFF ),

                .INTR_N ( INTR_N ),

                .PERRQ_N ( PERRQ_N ),

                .SERRQ_N ( SERRQ_N ),

                .KEEPOUT ( KEEPOUT ),

                .CSR ( CSR ),

                .SUB_DATA ( SUB_DATA ),

                .CFG ( CFG ),

                .RST ( RST ),

                .CLK ( CLK )

                );





  // Instantiation of the configuration module



  cfg CFG_INST (

                .CFG ( CFG )

                );





  // Instantiation of userapp back-end application template



  userapp USER_APP (

//               .FRAMEQ_N ( FRAMEQ_N ),

//                .TRDYQ_N ( TRDYQ_N ),

//                .IRDYQ_N ( IRDYQ_N ),

//                .STOPQ_N ( STOPQ_N ),

//                .DEVSELQ_N ( DEVSELQ_N ),

                .ADDR ( ADDR ),

                .ADIO ( ADIO ),

//                .CFG_VLD ( CFG_VLD ),

//                .CFG_HIT ( CFG_HIT ),

                .C_TERM ( C_TERM ),

                .C_READY ( C_READY ),

//                .ADDR_VLD ( ADDR_VLD ),

                .BASE_HIT ( BASE_HIT ),

                .S_TERM ( S_TERM ),

                .S_READY ( S_READY ),

                .S_ABORT ( S_ABORT ),

                .S_WRDN ( S_WRDN ),

                .S_SRC_EN ( S_SRC_EN ),

                .S_DATA_VLD ( S_DATA_VLD ),

                .S_CBE ( S_CBE ),

//                .PCI_CMD ( PCI_CMD ),

                .REQUEST ( REQUEST ),

                .REQUESTHOLD ( REQUESTHOLD ),

                .COMPLETE ( COMPLETE ),

                .M_WRDN ( M_WRDN ),

                .M_READY ( M_READY ),

//                .M_SRC_EN ( M_SRC_EN ),

//                .M_DATA_VLD ( M_DATA_VLD ),

                .M_CBE ( M_CBE ),

//                .TIME_OUT ( TIME_OUT ),

                .CFG_SELF ( CFG_SELF ),

//                .M_DATA ( M_DATA ),

//                .DR_BUS ( DR_BUS ),

//                .I_IDLE ( I_IDLE ),

//                .M_ADDR_N ( M_ADDR_N ),

//                .IDLE ( IDLE ),

//                .B_BUSY ( B_BUSY ),

                .S_DATA ( S_DATA ),

//                .BACKOFF ( BACKOFF ),

                .INTR_N ( INTR_N ),

//                .PERRQ_N ( PERRQ_N ),

//                .SERRQ_N ( SERRQ_N ),

                .KEEPOUT ( KEEPOUT ),

//                .CSR ( CSR ),

                .SUB_DATA ( SUB_DATA ),

//                .CFG ( CFG ),

                .RST ( reset ),

                .CLK ( CLK ),



				.SD_ADDR (SDRAM_A),

				.SD_BS (SDRAM_BA),

			//	.SD_CLK (),

				.SD_CKE (SDRAM_CKE),

				.SD_CS_N (SDRAM_CS_L),

				.SD_RAS_N (SDRAM_RAS_L),

				.SD_CAS_N (SDRAM_CAS_L),

				.SD_WE_N (SDRAM_WE_L),

				.SD_DQM (SDRAM_DQM),

				.SD_DATA (SDRAM_DQ),

            .DISPLAY1A (DISPLAY1A),

            .DISPLAY1B (DISPLAY1B),

            .DISPLAY1C (DISPLAY1C),

            .DISPLAY1D (DISPLAY1D),

            .DISPLAY1E (DISPLAY1E),

            .DISPLAY1F (DISPLAY1F),

            .DISPLAY1G (DISPLAY1G),

            .DISPLAY2A (DISPLAY2A),

            .DISPLAY2B (DISPLAY2B),

            .DISPLAY2C (DISPLAY2C),

            .DISPLAY2D (DISPLAY2D),

            .DISPLAY2E (DISPLAY2E),

            .DISPLAY2F (DISPLAY2F),

            .DISPLAY2G (DISPLAY2G),

            .LED_USER (LED_USER),

            .DIP (DIP), 

            .PUSH_USER1    (PUSH_USER1),

            .SIN           (sin),

            .SOUT          (sout),

            .SIG_IN        (SIG_IN),

            .SIG_OUT       (SIG_OUT)

                );





// A DLL added to help meet SDRAM timing

CLKDLL dll_sdram (.CLKIN(CLK), .CLKFB(CLK_FB_O), .RST(RST),

              .CLK0(CLK_FB), .CLK90(), .CLK180(SDRAM_CLK_OUT), .CLK270(),

              .CLK2X(), .CLKDV(), .LOCKED(LOCKED));

//assign SDRAM_CLK = CLK_FB_O;

BUFG bufg_sdram (.I(SDRAM_CLK_OUT), .O(SDRAM_CLK));

BUFG bufg_gclk (.I(CLK_FB), .O(CLK_FB_O));



endmodule

