-------------------------------------------------------------------------

--

-- File name   : phase_control.vhd

--             :  

-- Title       : 

--             :  

-- Functional  : 

-- Description : block for monitoring pci transactions and controlling 

--             : state of ready and terminate signals for transactions

--             : 

-- Created On  : 8/7/00   

--             :

-- Testbed     :

-- Requirements: <testing requirements -- explain relation to spec.>

--             :

-- Coding      :  

-- Standard    : Active low signals are identified with '_n' or '_N'.

--             : Only one entity-architecture pair per file.

--             :

-- Synthesis   :

-- Standard    : All outputs registered when possible.

--             :

-- Synthesis   :

-- Warnings    : <NO warnings without written explanation>

--             :

-- Device      :

-- Utilization : <est. clb count, tbufs, dffs, etc.>

--             :

-- Limitations : For use in Xilinx FPGA

--             : 

-- Developers  : Tom Curran

--             : Memec Design

--             : 30 Nagog Park

--             : Acton, MA  01720

--             : Ph 978.266.9193

--             : Fx 978.266.9194

-- ----------------------------------------------------------------------

-- Revision History :

-- ----------------------------------------------------------------------

--   Version   | Author          | Changes Made:      | Mod. Date:

--      1.0    | tom curran      | First write        | 8/7/00      

-- ----------------------------------------------------------------------



library IEEE;

use IEEE.std_logic_1164.all;

use ieee.std_logic_unsigned.all;



entity phase_control is

port (

   rst            : in std_logic;            -- Global Reset

   clk            : in std_logic;            -- PCLK

   busy           : in std_logic;

   base_hit       : in std_logic;

   bar_wr         : in std_logic;

   bar_rd         : in std_logic;

   s_data         : in std_logic;

   w_ready        : out std_logic;

   r_ready        : out std_logic;

   w_term         : out std_logic;

   r_term         : out std_logic

   );

end phase_control;

      

architecture rtl of phase_control is



   ----------------

   -- define types

   ----------------

   type TERM_TYPE is ( sIDLE, sRETRY, sWAIT, sEND);



   ------------------

   -- define signals

   ------------------

   signal term_state : TERM_TYPE;                     -- PCI Write Termination State Machine

   signal delay1 : std_logic;

   signal end_cycle : std_logic;

   

begin



   r_ready <= end_cycle;

   r_term <= end_cycle;

   

   -- end cycle logic

   read_cycle_logic : process(clk, rst)

   begin

      if rst = '1' then

         delay1 <= '0';

         end_cycle <= '0';

      elsif rising_edge(clk) then

         -- generate one clk wide pulse

         if end_cycle = '0' then

            delay1 <= bar_rd;

            end_cycle <= delay1;

         else

            delay1 <= '0';

            end_cycle <= '0';

         end if;

      end if;

   end process;



   -- Termination logic will force retry if busy writing to

   -- external bus.  It will also force retry if FIFO is full at

   -- the start of a cycle.  If FIFO becomes full during a write

   -- cycle, it will disconnect with data

   write_cycle_logic : process(clk, rst)

   begin

      if rst = '1' then

         w_term <= '0';

         w_ready <= '0';

         term_state <= sIDLE;

      elsif rising_edge(clk) then

         case term_state is

         when sIDLE =>

            -- if busy force RETRY

            if base_hit = '1' and busy = '1' then

               term_state <= sRETRY;

               w_term <= '1';

               w_ready <= '0';

            elsif bar_wr = '1' and busy = '0' then

               -- normal mode

               term_state <= sWAIT;

               w_term <= '0';

               w_ready <= '1';

            end if;



			when sRETRY =>

            -- state in RETRY until data portion of PCI cycle is over

				if s_data = '0' then

					-- wait mode

					term_state <= sIDLE;

					w_term <= '0';

					w_ready <= '0';

				else

					term_state <= sRETRY;

				end if;



         when sWAIT =>

            if s_data = '0'then

               -- initiator has terminated PCI cycle

               -- goto wait mode

               term_state <= sIDLE;

               w_term <= '0';

               w_ready <= '0';

            else

               term_state <= sWAIT;

            end if;

      

         when sEND =>

            -- Wait for end of PCI cycle

            if s_data = '0'then

               term_state <= sIDLE;

               w_term <= '0';

               w_ready <= '0';

            else

               term_state <= sEND;

            end if;

         end case;

      end if;

   end process;

end;