/*--

-- File name 	: 	REGISTER_MODULE.v

--             :

-- Title       :  Register Module

--             :

-- Purpose     :  This is the register interface for the sample SDRAM design

--					:

--             :

--             :

--             :

-- Created On  : 08/23/00

--             :

-- Comments    : Design done by Insight Design Services

--             :

--					:

--					:

--             :

-- Conventions :

--					: Signal names ending in _int indicate the internal version of an output signal

--             : Tab size: 4

--   				:

--   				:

--   				:

-- Limitations : For use in Xilinx Virtex E FPGA

--   				:

-- Developers  : Duncan Allen

-- ----------------------------------------------------------------------

-- Revision History :

-- ----------------------------------------------------------------------

--   Version No:| Author    	| Changes Made: 						| Mod. Date:

--     v1.0     | da  			| First Written						| 10/11/01

-- ------------------------------------------------------------------------*/

`timescale 1 ns/1 ps





module REGISTER_MODULE (

            CLK,

            RESET,



            REGISTER_WRITE_HIT,

            REGISTER_READ_HIT,

            S_DATA,

            ADDRESS,



            REGISTER_WRITE_DATA,

            REGISTER_READ_ENABLE,

            REGISTER_READ_DATA,



            REGISTER_READY,

            REGISTER_TERM,

            REGISTER_ABORT,

            

            // UART CORE interface

            UART_CTRL_REG,

            UART_WEN_TXFIFO,

            UART_WDATA_TXFIFO,

            UART_REN_RXFIFO,

            UART_RDATA_RXFIFO,   

            UART_FIFO_STATUS,

            UART_RX_OF,

            

            // These are added for diagnostic purposes

//            WRITE_PUSH,

//            READ_PUSH,

//            SDRAM_DATA,

            

            LED_USER,

            SEVEN_SEG1,

            SEVEN_SEG2,

            DIP,        

            PUSH_USER1,

            SIG_IN,

            SIG_OUT,

            INTR_N

            );





   input CLK;

   input RESET;

   input REGISTER_WRITE_HIT;

   input REGISTER_READ_HIT;

   input S_DATA;

   input [3:0]ADDRESS;



   input [31:0] 	REGISTER_WRITE_DATA;

   output   		REGISTER_READ_ENABLE;

   output [31:0]  REGISTER_READ_DATA;



   output REGISTER_READY;

   output REGISTER_TERM;

   output REGISTER_ABORT;



   output [31:0]  UART_CTRL_REG;

   output         UART_WEN_TXFIFO;

   output [7:0]   UART_WDATA_TXFIFO;

   output         UART_REN_RXFIFO;

   input  [7:0]   UART_RDATA_RXFIFO; 

   input [15:0]   UART_FIFO_STATUS;

   input          UART_RX_OF;

            

//   input WRITE_PUSH;

//   input READ_PUSH;

//   input [31:0] SDRAM_DATA;

   

   output         LED_USER;

   input  [7:0]   DIP;         

   input          PUSH_USER1;

   input [53:0]   SIG_IN;

   output [53:0]  SIG_OUT;

   output [3:0] SEVEN_SEG1, SEVEN_SEG2;

   output         INTR_N;

   

   reg [31:0]  UART_CTRL_REG;

   reg         UART_WEN_TXFIFO;

   reg         UART_REN_RXFIFO;

            

   wire register_read;

   wire register_write;



   wire [31:0] register_0;

   wire [31:0] register_1;

   wire [31:0] register_2;

   wire [31:0] register_3;

   wire [31:0] register_4;

   wire [31:0] register_5;

   wire [31:0] register_6;

   wire [31:0] register_7;

   wire [31:0] register_8;

   wire [31:0] register_9;

   wire [31:0] uart_status_reg;

   

   reg         reg_0;

   reg [7:0]   reg_1;

   reg [31:0]  reg_2;

   reg [21:0]  reg_3;    

   reg [4:0]   int_ctrl_reg;     

   wire        int_tx_en, int_rx_en, int_pb_en, int_dip_en, intr_en;

   reg         int_tx, int_rx, int_pb, int_dip, intr;

   reg         rd_int_status, rd_int_status_dly;

   wire        clr_int;      

   reg         tx_fifo_wen_dly;

   reg         rx_fifo_ren_dly1;

   wire        rx_fifo_ren;

   reg         uart_rxfifo_overflow;

   reg         rd_uart_status, rd_uart_status_dly;

   reg         rx_intr;

   wire        tx_intr;

   wire        rx_rdy;

   wire        tx_rdy;

   



   reg [31:0] read_data;



   reg       PB_d1, PB_d2, PB_d3;

   reg [7:0] DIP_d1, DIP_d2, DIP_d3;

   reg [53:0] SIG_IN_d1, SIG_IN_d2;



assign UART_WDATA_TXFIFO = REGISTER_WRITE_DATA[7:0];   

assign LED_USER = reg_0;   

assign SEVEN_SEG1 = reg_1 [3:0];

assign SEVEN_SEG2 = reg_1 [7:4];

assign SIG_OUT = {reg_3, reg_2};



assign register_0 = {31'b0, reg_0};

assign register_1 = {24'b0, reg_1};

assign register_2 = reg_2;

assign register_3 = {10'b0, reg_3};

assign register_4 = SIG_IN_d2[31:0];

assign register_5 = {10'b0, SIG_IN_d2[53:32]};

assign register_6 = {24'b0, DIP_d3};

assign register_7 = {27'b0, int_ctrl_reg};

assign register_8 = {27'b0, int_tx, int_rx, int_pb, int_dip, intr};

assign register_9 = {31'b0, PB_d3}; 

assign uart_status_reg = {UART_FIFO_STATUS,13'b0,uart_rxfifo_overflow, rx_rdy, tx_rdy};



always @ (posedge RESET or posedge CLK)

   if (RESET) rx_intr <= 0;

   else if ((UART_FIFO_STATUS[14:7] > UART_CTRL_REG[23:16]) || UART_FIFO_STATUS[15])

      rx_intr <= 1;

   else if ((UART_FIFO_STATUS[14:7] == UART_CTRL_REG[23:16]) && !UART_FIFO_STATUS[15])

      rx_intr <= 0;



assign tx_intr = UART_FIFO_STATUS[0];

assign rx_rdy = ~UART_FIFO_STATUS[2];

assign tx_rdy = ~UART_FIFO_STATUS[1];

   

   REGISTER_HIT_MACHINE iREG_MACHINE (

            .REGISTER_READ_HIT	(REGISTER_READ_HIT),

				.REGISTER_WRITE_HIT	(REGISTER_WRITE_HIT),

				.S_DATA					(S_DATA),

				.CLK						(CLK),

				.RESET					(RESET),

				.REGISTER_READ			(register_read),

				.REGISTER_WRITE		(register_write),

				.REGISTER_CLEAR		(),

				.REGISTER_ABORT		(REGISTER_ABORT),

				.REGISTER_TERM			(REGISTER_TERM),

				.REGISTER_READY      (REGISTER_READY)

				);





   // Process to load register_0

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         reg_0 <= 0;

      else if (register_write && (ADDRESS ==4'b0000) && S_DATA)

         reg_0 <= REGISTER_WRITE_DATA[0];

      end



   // Process to load register_1

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         reg_1 <= 8'h0;

      else if (register_write && (ADDRESS == 4'b0001) && S_DATA)

         reg_1 <= REGISTER_WRITE_DATA[7:0];

      end



      // Process to load register_2

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         reg_2 <= 32'h0;

      else if (register_write && (ADDRESS == 4'b010) && S_DATA)

         reg_2 <= REGISTER_WRITE_DATA;

      end



      // Process to load register_3

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         reg_3 <= 22'h0;

      else if (register_write && (ADDRESS == 4'b0011) && S_DATA)

         reg_3 <= REGISTER_WRITE_DATA[21:0];

      end

      

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         int_ctrl_reg <= 'h0;

      else if (register_write && (ADDRESS == 4'b0111) && S_DATA)

         int_ctrl_reg <= REGISTER_WRITE_DATA[4:0];

      end                   

      

assign int_tx_en = int_ctrl_reg[4];      

assign int_rx_en = int_ctrl_reg[3];      

assign int_pb_en = int_ctrl_reg[2];      

assign int_dip_en = int_ctrl_reg[1];      

assign intr_en = int_ctrl_reg[0];      

      



// UART registers and their control signals

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         UART_CTRL_REG <= 'h0;

      else if (register_write && (ADDRESS == 4'b1110) && S_DATA)

         UART_CTRL_REG <= REGISTER_WRITE_DATA;

      end    

      

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET) begin

         tx_fifo_wen_dly <= 0;

         UART_WEN_TXFIFO <= 0;

         end

      else if (register_write && (ADDRESS == 4'b1100) && S_DATA) begin

         tx_fifo_wen_dly <= 1;                            

         UART_WEN_TXFIFO <= ~ tx_fifo_wen_dly;

         end

      else begin

         tx_fifo_wen_dly <= 0;

         UART_WEN_TXFIFO <= 0;

         end

      end                           



assign rx_fifo_ren = (register_read && (ADDRESS == 4'b1101))? 1 : 0;

   always @ (posedge CLK or posedge RESET)

      if (RESET) begin

         rx_fifo_ren_dly1 <= 0;

         UART_REN_RXFIFO <= 0;

         end

      else begin

         rx_fifo_ren_dly1 <= rx_fifo_ren;

         UART_REN_RXFIFO <= rx_fifo_ren_dly1 & (~ rx_fifo_ren);

         end

      

         



// debounce      

	always @(posedge CLK or posedge RESET)

      begin

      if (RESET)

         begin    

         PB_d1 <= 0;

         PB_d2 <= 0;

         PB_d3 <= 0;

         DIP_d1 <= 8'h0;

         DIP_d2 <= 8'h0;

         DIP_d3 <= 8'h0;

         SIG_IN_d1 <= 54'h0;

         SIG_IN_d2 <= 54'h0;

         end

      else

         begin      

         PB_d1 <= PUSH_USER1;

         PB_d2 <= PB_d1;

         PB_d3 <= PB_d2;

         DIP_d1 <= DIP;

         DIP_d2 <= DIP_d1;

         DIP_d3 <= DIP_d2;

         SIG_IN_d1 <= SIG_IN;

         SIG_IN_d2 <= SIG_IN_d1;

         end

      end

      

// interrupt logics      

always @ (posedge CLK or posedge RESET)      

   if (RESET) int_tx <= 0;

   else if (clr_int) int_tx <= 0;

   else if (tx_intr && int_tx_en) int_tx <= 1;

      

always @ (posedge CLK or posedge RESET)      

   if (RESET) int_rx <= 0;

   else if (clr_int) int_rx <= 0;

   else if (rx_intr && int_rx_en) int_rx <= 1;

      

always @ (posedge CLK or posedge RESET)      

   if (RESET) int_pb <= 0;

   else if (clr_int) int_pb <= 0;

   else if ((!PB_d3) && (!PB_d2) && int_pb_en) int_pb <= 1;

      

always @ (posedge CLK or posedge RESET)      

   if (RESET) int_dip <= 0;

   else if (clr_int) int_dip <= 0;

   else if ((DIP_d3 != DIP_d2) && int_dip_en) int_dip <= 1;

      

always @ (posedge CLK or posedge RESET)      

   if (RESET) intr <= 0;

   else if (clr_int || (!intr_en)) intr <= 0;

   else intr <= int_pb | int_dip | int_tx | int_rx;



assign INTR_N = ~intr;   



always @ (posedge CLK or posedge RESET)

   if (RESET) rd_int_status <= 0;

   else if (register_read && (ADDRESS == 4'b1000)) rd_int_status <= 1;

   else rd_int_status <= 0;

      

always @ (posedge CLK or posedge RESET)

   if (RESET) rd_int_status_dly <= 0;

   else rd_int_status_dly <= rd_int_status;

      

assign clr_int = rd_int_status_dly & (~rd_int_status);      

      

      

always @ (posedge CLK or posedge RESET)

   if (RESET) rd_uart_status <= 0;

   else if (register_read && (ADDRESS == 4'b1111)) rd_uart_status <= 1;

   else rd_uart_status <= 0;

      

always @ (posedge CLK or posedge RESET)

   if (RESET) rd_uart_status_dly <= 0;

   else rd_uart_status_dly <= rd_uart_status;

      

always @ (posedge CLK or posedge RESET)

   if (RESET) uart_rxfifo_overflow <= 0;

   else if (rd_uart_status_dly && !rd_uart_status) uart_rxfifo_overflow <= 0;

   else if (UART_RX_OF) uart_rxfifo_overflow <= 1;

      

      

// read data output      

always @ (ADDRESS or register_0 or register_1 or register_2 or register_3 or 

         register_4 or register_5 or register_6 or register_7 or register_8 or

         register_9 or UART_RDATA_RXFIFO or UART_CTRL_REG or uart_status_reg)

   begin

   case (ADDRESS)

      4'b0000 : read_data = register_0;

      4'b0001 : read_data = register_1;

      4'b0010 : read_data = register_2;

      4'b0011 : read_data = register_3;

      4'b0100 : read_data = register_4;

      4'b0101 : read_data = register_5;

      4'b0110 : read_data = register_6;

      4'b0111 : read_data = register_7;

      4'b1000 : read_data = register_8;

      4'b1001 : read_data = register_9;

      4'b1101 : read_data = {24'h0,UART_RDATA_RXFIFO};

      4'b1110 : read_data = UART_CTRL_REG;

      4'b1111 : read_data = uart_status_reg;

      default : read_data = register_0;

   endcase

end



assign REGISTER_READ_DATA = read_data;

assign REGISTER_READ_ENABLE = register_read;



endmodule