//--------------------------------------------------------------------------------------------------

//             :

// File name   : BOARDTEST_TB_TOP.V

//             :

// Title       : Top level of the S2PCI SDRAM controller testbench

//             :

// Comments    :

//             :

//             :

// Conventions : - Port names are 'UPPER' case.

//             : - Internal signals and variables are 'lower' case.

//             : - Active low signals are identified with '_n' or '_N'.

//             : - instances begin with lower case 'i', rest upper case

//             : - parameters begin with lower case 'p', rest upper case

//             : - tasks begin with lower case 't', rest upper case

//             :

//             : Tab setting = 3

//             : Right Margin = 100

//             :

// Limitations : For use in testbench only

//             :

// Developers  : Majid Kalsi

//             :

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//   Version No | Author    | Changes Made:                                            | Mod. Date:

//     v1.0     | da        | First Written          												| 27/08/01

//--------------------------------------------------------------------------------------------------





// This module will test the S2PCI userapp



`timescale 1 ns/1 ps



module BOARDTEST_TB_TOP ();



wire [31:0]	ad;

wire [3:0]	cbe;

wire		par;

wire		frame_n;

wire		trdy_n;

wire		irdy_n;

wire		stop_n;

wire		devsel_n;

wire		idsel;

wire		intr_a;

wire		perr_n;

wire		serr_n;

wire		req_n;

wire		gnt_n;

wire		rst_n;

wire		pclk;



wire [12:0]	sd_addr;

wire [1:0]	sd_bs;

wire		sd_clk;

wire		sd_cke;

wire 		sd_cs_n;

wire		sd_ras_n;

wire 		sd_cas_n;

wire		sd_we_n;

wire [3:0]	sd_dqm;

wire [31:0] sd_data;



wire [3:0]  icmd;

wire 		icmd_ack;

wire [99:0]	istate;

wire		ibusy;

wire [3:0]	tcmd;

wire		tcmd_ack;

wire [99:0] tstate;

wire		tbusy;



wire [3:0] 	irdy_initial_latency;

wire [3:0]  trdy_initial_latency;



wire		insert_wait_states;

wire [3:0]	initial_value;

wire [3:0]	term_cnt;

wire [3:0]	mod_amt;

wire [3:0] 	result;

wire [3:0]	duration;      



wire [7:0] DIP;

wire [53:0] SIG_IN;

wire [53:0] SIG_OUT;   

wire [6:0]  DISPLAY1, DISPLAY2;

wire        FPGA_RESETn; 

wire        RXD;

wire        reset;

wire        clock;

reg [159:0] rxd_dly;

wire          TXD;



wire        logic0, logic1;



wire		end_sim;



// Instantiate the sdram Module

 sdram_top UUT	(

                .AD (ad),             // PCI ports -- do not modify names!

                .CBE (cbe),

                .PAR (par),

                .FRAME_N (frame_n),

                .TRDY_N (trdy_n),

                .IRDY_N (irdy_n),

                .STOP_N (stop_n),

                .DEVSEL_N (devsel_n),

                .IDSEL (idsel),

                .INTR_A (intr_a),

                .PERR_N (perr_n),

                .SERR_N (serr_n),

                .REQ_N  (req_n),

                .GNT_N  (gnt_n),

                .RST_N	(rst_n),

                .PCLK	(pclk),

				// SDRAM signals

				.SDRAM_A 		(sd_addr[10:0]),

				.SDRAM_BA 		(sd_bs),

				.SDRAM_CLK 		(sd_clk),

				.SDRAM_CKE 		(sd_cke),

				.SDRAM_CS_L 	(sd_cs_n),

				.SDRAM_RAS_L	(sd_ras_n),

				.SDRAM_CAS_L 	(sd_cas_n),

				.SDRAM_WE_L 	(sd_we_n),

				.SDRAM_DQM 		(sd_dqm),

				.SDRAM_DQ 		(sd_data),  

            

            .DISPLAY1A (DISPLAY1[6]),

            .DISPLAY1B (DISPLAY1[5]),

            .DISPLAY1C (DISPLAY1[4]),

            .DISPLAY1D (DISPLAY1[3]),

            .DISPLAY1E (DISPLAY1[2]),

            .DISPLAY1F (DISPLAY1[1]),

            .DISPLAY1G (DISPLAY1[0]),

            .DISPLAY2A (DISPLAY2[6]),

            .DISPLAY2B (DISPLAY2[5]),

            .DISPLAY2C (DISPLAY2[4]),

            .DISPLAY2D (DISPLAY2[3]),

            .DISPLAY2E (DISPLAY2[2]),

            .DISPLAY2F (DISPLAY2[1]),

            .DISPLAY2G (DISPLAY2[0]),

            

				.LED_USER		(led_user),

            .DIP           (DIP),   

            .PUSH_USER1    (PUSH_USER1),

            .FPGA_RESETN   (FPGA_RESETn),

            .SIG_IN        (SIG_IN),

            .SIG_OUT       (SIG_OUT),

            .RXD           (RXD),

            .TXD           (TXD)   

              

	);     

   

assign SIG_IN = SIG_OUT;  

assign PUSH_USER1 = led_user;           

assign FPGA_RESETn = 1;



assign TXD = rxd_dly[159];   



assign logic0 = 0;

assign logic1 = 1;



seg2bin seg1 (.seg (DISPLAY1), .bin(DIP[3:0]));

seg2bin seg2 (.seg (DISPLAY2), .bin(DIP[7:4]));



// Instantiate the SDRAMs

 mt48lc2m32b2  sdram_0 (

 	.Dq 	(sd_data [31:0]),

 	.Addr 	(sd_addr[10:0]),

	.Ba 	(sd_bs),

	.Clk 	(sd_clk),

	.Cke 	(sd_cke),

	.Cs_n 	(sd_cs_n),

	.Ras_n 	(sd_ras_n),

	.Cas_n 	(sd_cas_n),

	.We_n 	(sd_we_n),

	.Dqm 	(sd_dqm)

	);





// Instantiate the PCI model

 SDRAM_TB_PCI iPCI_MODULE (

      // PCI signals

	.AD (ad),

   	.CBE (cbe),

   	.PAR (par),

   	.FRAME_N (frame_n),

   	.TRDY_N (trdy_n),

   	.IRDY_N (irdy_n),

   	.STOP_N (stop_n),

   	.DEVSEL_N (devsel_n),

   	.dut_IDSEL (idsel),

    .dut_INTR_A (intr_a),

   	.PERR_N (perr_n),

   	.SERR_N (serr_n),

   	.dut_REQ_N (req_n),

   	.dut_GNT_N (gnt_n),



   	.RST_N (rst_n),

   	.PCLK (pclk),



	// Testbench Control Signals

    // command/control interface

   	.ICMD (icmd),                   // Initiator Command

   	.ICMD_ACK (icmd_ack),               // Initiator Command Acknowledge

   	.ISTATE (istate),                 // Initiator State

   	.IBUSY (ibusy),                  // Initiator Busy

   	.TCMD (tcmd),                   // Target Command

   	.TCMD_ACK (tcmd_ack),               // Target Command Acknowledge

   	.TSTATE (tstate),                 // Target State

   	.TBUSY (tbusy),                  // Target Busy

   	.END_SIM (end_sim),                // End of Simulation Flag



   	// Initial Latency Control

   	.IRDY_INITIAL_LATENCY (irdy_initial_latency),   // IRDY Initial latency, min 0, max 7

   	.TRDY_INITIAL_LATENCY (trdy_initial_latency),   // TRDY Initial latency, min 0, max 15



   	// Wait State Control, see description above

   	.INSERT_WAITS (insert_waits),           // Insert Wait States Flag

   	.INITIAL_VALUE (initial_value),          // initial value of "count"

   	.TERM_CNT (term_cnt),               // termination count

   	.MOD_AMT (mod_amt),                // modulus amount

   	.RESULT  (result) ,                 // test for mod function result

   	.DURATION (duration)               // duration of waits, min 1, max 7

   );



   // Instantiate the TB control module

 SDRAM_TB_CONTROL iCONTROL   (

    // PCI control signals

    .pci_ICMD (icmd),

   	.pci_ICMD_ACK (icmd_ack),

   	.pci_ISTATE (istate),

   	.pci_IBUSY (ibusy),

   	.pci_TCMD (tcmd),

   	.pci_TCMD_ACK (tcmd_ack),

   	.pci_TSTATE (tstate),

   	.pci_TBUSY (tbusy),



    .pci_RST_N (rst_n),



    .pci_CLK (pclk),

   	// PCI Initial Latency Control

   	.pci_IRDY_INITIAL_LATENCY (irdy_initial_latency),

   	.pci_TRDY_INITIAL_LATENCY (trdy_initial_latency),



   	// PCI Wait State Control

   	.pci_INSERT_WAITS (insert_waits),

   	.pci_INITIAL_VALUE (initial_value),

   	.pci_TERM_CNT (term_cnt),

   	.pci_MOD_AMT (mod_amt),

   	.pci_RESULT (result),

   	.pci_DURATION (duration),



      // General Testbench Control Signals



     .END_SIM (end_sim)

   );

   

assign reset = ~rst_n;

assign clock = ~pclk;

always @ (posedge reset or posedge clock)

   if (reset) rxd_dly <= 160'hFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF;

   else begin

      rxd_dly[0] <= RXD;

      rxd_dly[159:1] <= rxd_dly[158:0];

   end

   

endmodule