//--------------------------------------------------------------------------------------------------

//             :

// File name   : PCI_DEVICE_DEFINES.V

//             :

// Title       : Generic PCI device globals for PCI testbench

//             :

// Comments    :

//             :

//             :

//             :

//             :

//             :

//             :

//             :

//             :

// Conventions : - Port names are 'UPPER' case.

//             : - Internal signals and variables are 'lower' case.

//             : - Active low signals are identified with '_n' or '_N'.

//             : - instances begin with lower case 'i', rest upper case

//             : - parameters begin with lower case 'p', rest upper case

//             : - tasks begin with lower case 't', rest upper case

//             :

//             : Tab setting = 3

//             : Right Margin = 100

//             :

// Limitations : For use in testbench only

//             :

// Developers  : Majid Kalsi

//             :

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//   Version No | Author    | Changes Made:                                            | Mod. Date:

//     v1.0     | mk        | First write                                              | 00/00/00

//--------------------------------------------------------------------------------------------------







// BAR sizes in bytes for 32 bit Addressing

`define SIZE2G       28'h8000_000

`define SIZE1G       28'hC000_000

`define SIZE512M     28'hE000_000

`define SIZE256M     28'hF000_000

`define SIZE128M     28'hF800_000

`define SIZE64M      28'hFC00_000

`define SIZE32M      28'hFE00_000

`define SIZE16M      28'hFF00_000

`define SIZE8M       28'hFF80_000

`define SIZE4M       28'hFFC0_000

`define SIZE2M       28'hFFE0_000

`define SIZE1M       28'hFFF0_000

`define SIZE512K     28'hFFF8_000

`define SIZE256K     28'hFFFC_000

`define SIZE128K     28'hFFFE_000

`define SIZE64K      28'hFFFF_000

`define SIZE32K      28'hFFFF_800

`define SIZE16K      28'hFFFF_C00

`define SIZE8K       28'hFFFF_E00

`define SIZE4K       28'hFFFF_F00

`define SIZE2K       28'hFFFF_F80

`define SIZE1K       28'hFFFF_FC0

`define SIZE512      28'hFFFF_FE0

`define SIZE256      28'hFFFF_FF0

`define SIZE128      28'hFFFF_FF8

`define SIZE64       28'hFFFF_FFC

`define SIZE32       28'hFFFF_FFE

`define SIZE16       28'hFFFF_FFF



// BAR sizes in bytes for 64 bit Addressing

`define SIZE4G       32'hFFFFFFFF

`define SIZE8G       32'hFFFFFFFE

`define SIZE16G      32'hFFFFFFFC

`define SIZE32G      32'hFFFFFFF8

`define SIZE64G      32'hFFFFFFF0

`define SIZE128G     32'hFFFFFFE0

`define SIZE256G     32'hFFFFFFC0

`define SIZE512G     32'hFFFFFF80

`define SIZE1T       32'hFFFFFF00

`define SIZE2T       32'hFFFFFE00

`define SIZE4T       32'hFFFFFC00

`define SIZE8T       32'hFFFFF800

`define SIZE16T      32'hFFFFF000

`define SIZE32T      32'hFFFFE000

`define SIZE64T      32'hFFFFC000

`define SIZE128T     32'hFFFF8000

`define SIZE256T     32'hFFFF0000

`define SIZE512T     32'hFFFE0000

`define SIZE1P       32'hFFFC0000

`define SIZE2P       32'hFFF80000

`define SIZE4P       32'hFFF00000

`define SIZE8P       32'hFFE00000

`define SIZE16P      32'hFFC00000

`define SIZE32P      32'hFF800000

`define SIZE64P      32'hFF000000

`define SIZE128P     32'hFE000000

`define SIZE256P     32'hFC000000

`define SIZE512P     32'hF8000000

`define SIZE1E       32'hF0000000

`define SIZE2E       32'hE0000000

`define SIZE4E       32'hC0000000

`define SIZE8E       32'h80000000



`define GT_2G        28'h0000_000      // Greater than 2 Gigabytes

`define LT_4G        32'hFFFFFFFF      // Less than 4 Gigabytes



// BAR parameters

`define IO_SPACE     1'b1

`define MEM_SPACE    1'b0

`define PREFETCH     1'b1

`define NOFETCH      1'b0



// Timing Constants

`define CLK33MHZ     0

`define CLK66MHZ     1

`define TDEL_66      6.0

`define TDEL_33      11.0



// Bus Width Constants

`define BIT32        1

`define BIT64        2



// DEVSEL Initial Latency

`define FAST         0

`define MEDIUM       1

`define SLOW         2

`define BRIDGE       3



// PCI cycle commands

`define INT_ACK      4'b0000

`define SPECIAL      4'b0001

`define IO_RD        4'b0010

`define IO_WR        4'b0011

`define MEM_RD       4'b0110

`define MEM_WR       4'b0111

`define CFG_RD       4'b1010

`define CFG_WR       4'b1011

`define MRM          4'b1100     // Memory Read Multiple

`define DAC          4'b1101     // Dual Address Cycle

`define MRL          4'b1110     // Memory Read Line

`define MWI          4'b1111     // Memory Write & Invalidate



// Initiator commands

`define NONE         0

`define TEST1        4'b0001//1

`define TEST2        4'h2//2

`define TEST3        4'h3//3

`define TEST4        4'h4//4

`define TEST5        4'h5//5

`define TEST6        4'h6//6

`define TEST7        4'h7//7

`define TEST8        4'h8//8

`define TEST9        4'h9//9



// Target commands

//`define NONE         0

`define DWD          1

`define DWOD         2

`define MABORT       3

`define TABORT       4

`define PAR_ERR      5

`define SYS_ERR      6

`define RANDOM       7



// Termination Types

`define TERM_NORM32  1

`define TERM_NORM64  2

`define TERM_MABORT  3

`define TERM_TABORT  4

`define TERM_DWD32   5

`define TERM_DWD64   6

`define TERM_DWOD    7

`define TERM_ERROR   8





// PCI configuration Header Address Space

`define HDR_CSR         32'h00000004

`define HDR_LAT_TMR     32'h0000000C

`define HDR_BAR0        32'h00000010

`define HDR_BAR1        32'h00000014

`define HDR_BAR2        32'h00000018

`define HDR_BAR3        32'h0000001C

`define HDR_BAR4        32'h00000020

`define HDR_BAR5        32'h00000024





// MISC

`define FALSE        1'b0

`define TRUE         1'b1

`define SIZEOFCHAR   8

