//--------------------------------------------------------------------------------------------------

//             :

// File name   : PCI_DEVICE_INITIATOR.V

//             :

// Title       : Generic PCI device Initiator Tasks

//             :

// Comments    :

//             :

//             :

// Conventions : - Port names are 'UPPER' case.

//             : - Internal signals and variables are 'lower' case.

//             : - Active low signals are identified with '_n' or '_N'.

//             : - instances begin with lower case 'i', rest upper case

//             : - parameters begin with lower case 'p', rest upper case

//             : - tasks begin with lower case 't', rest upper case

//             :

//             : Tab setting = 3

//             : Right Margin = 100

//             :

// Limitations : For use in testbench only

//             :

// Developers  : Majid Kalsi

//             :

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//   Version No | Author    | Changes Made:                                            | Mod. Date:

//     v1.0     | mk        | First write                                              | 00/00/00

//     v1.1     | da        | Modified to test S2PCI SDRAM controller interface          | 27/08/01

//--------------------------------------------------------------------------------------------------









//--------------------------------------------------------------------------------------------------

task tINIT_S2PCI;

//--------------------------------------------------------------------------------------------------

   reg [31:0] addr;

   reg [31:0] data;

   reg [63:0] data64;

   integer i;



   begin

   $display($time, " ns [%s INITIATOR] Begin Initialization of SDRAM_CONTROLLER FPGA....", pINST_NAME);

   $fdisplay(logfile, $time, " ns [%s INITIATOR] Begin Initialization of SDRAM_CONTROLLER FPGA....", pINST_NAME);



	IBUSY = `TRUE;

   ISTATE = "INIT S2PCI";



   //-----------------------------------------

   // (1) Setup command register to enable:

   // 	a. memory access

   // 	b. Bus Mastering

   //    c. Parity Error Response

   //    d. System Error Response

   //

   // (2) Clear status register

   //

   //-----------------------------------------

   $display($time, " ns [%s INITIATOR] Setup PCI Configuration Register...", pINST_NAME);

   $fdisplay(logfile, $time, " ns [%s INITIATOR] Setup PCI Configuration Register...", pINST_NAME);





	addr = (`HDR_CSR | 32'h20000000);

   data = 32'h000001C6;

   #(`BIT32) tSAC_SINGLE(`CFG_WR, 4'b0000, addr, data);



   // Read back and verify

   #(`BIT32) tSAC_SINGLE(`CFG_RD, 4'b0000, addr, data);

   $display ($time, "ns three");

   data = data & 32'h0000FFFF;

   if(data != 32'h000001C6) begin

      $display($time, " ns [%s INITIATOR] PCI Configuration Command register error!", pINST_NAME);

      $fdisplay(logfile, $time, " ns [%s INITIATOR] PCI Configuration Command register error!", pINST_NAME);



      $display($time, " ns \tExpected: 00000007H  Received %8HH", data);

      $fdisplay(logfile, $time, " ns \tExpected: 00000007H  Received %8HH", data);

      $stop(2);

   end



   //-----------------------------------------

   // Check BARs

   //-----------------------------------------

	for(i = 0; i <2; i = i + 1) begin

      $display($time, " ns [%s INITIATOR] Verifying BAR%d is 4MBytes...", pINST_NAME, i);

      $fdisplay(logfile, $time, " ns [%s INITIATOR] Verifying BAR%d is 4MBytes...", pINST_NAME, i);



		// write to Bar

		addr = (`HDR_BAR0 | 32'h20000000) + 4*i;

      data = 32'hFFFFFFFF;

      #(`BIT32) tSAC_SINGLE(`CFG_WR, 4'b0000, addr, data);



		// Read back Bar

		addr = (`HDR_BAR0 | 32'h20000000) + 4*i;

      #(`BIT32) tSAC_SINGLE(`CFG_RD, 4'b0000, addr, data);



		/*// Verify

      if(data != 32'hFFC0_0000) begin

         $display($time, " ns [%s INITIATOR] PCI Configuration BAR%d address range error!", pINST_NAME, i);

         $fdisplay(logfile, $time, " ns [%s INITIATOR] PCI Configuration BAR%d address range error!", pINST_NAME, i);



      	$display($time, " ns \tExpected: FFC0_0000H  Received %8HH", data);

      	$fdisplay(logfile, $time, " ns \tExpected: FFC0_0000H  Received %8HH", data);

         $stop(2);

      end */



      // Define starting Address for BAR address space

      $display($time, " ns [%s INITIATOR] defining start of BAR%d address space...", pINST_NAME, i);

      $fdisplay(logfile, $time, " ns [%s INITIATOR] defining start of BAR%d address space...", pINST_NAME, i);



		addr = (`HDR_BAR0 | 32'h20000000) + 4*i;

		//data =  {i,31'b0};

      if (i == 1)

         data = 32'h00000000;

      else

         data = 32'h10000000;

      #(`BIT32) tSAC_SINGLE(`CFG_WR, 4'b0000, addr, data);

   end



   //-----------------------------------------

   // Setup Latency Timer

   //-----------------------------------------

   $display($time, " ns [%s INITIATOR] Setting Latency Timer to Maximum...", pINST_NAME);

   $fdisplay(logfile, $time, " ns [%s INITIATOR] Setting Latency Timer to Maximum...", pINST_NAME);



	addr = (`HDR_LAT_TMR | 32'h20000000);

   data = 32'h0000FF00;

   #(`BIT32) tSAC_SINGLE(`CFG_WR, 4'b1101, addr, data);



   // Read back and verify

   #(`BIT32) tSAC_SINGLE(`CFG_RD, 4'b0000, addr, data);

   data = data & 32'h0000FF00;

   if(data != 32'h0000FF00) begin

      $display($time, " ns [%s INITIATOR] PCI Configuration Latency Timer Setup error!", pINST_NAME);

      $fdisplay(logfile, $time, " ns [%s INITIATOR] PCI Configuration Latency Timer Setup error!", pINST_NAME);



     	$display($time, " ns \tExpected: 0000FF00H  Received %8HH", data);

     	$fdisplay(logfile, $time, " ns \tExpected: 0000FF00H  Received %8HH", data);

      $stop(2);

   end



	$display($time, " ns [%s INITIATOR] Initialization of  FPGA End", pINST_NAME);

   $display("\n");

	$fdisplay(logfile, $time, " ns [%s INITIATOR] Initialization of  FPGA End", pINST_NAME);

   $fdisplay(logfile, "\n");



   //Set the interrupt active register to allow the tSERVICE_INTERRUPT to run

   interrupt_active = 1'b1;

   //tMEMORY_DUMP;

	IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask





//--------------------------------------------------------------------------------------------------

task tSERVICE_INTERRUPT;

//--------------------------------------------------------------------------------------------------

   begin

 

   end

endtask





//--------------------------------------------------------------------------------------------------

task tDO_TEST1;

//--------------------------------------------------------------------------------------------------

	reg [31:0] start_addr;

	reg [31:0] data_0;

	reg [31:0] data_1;

	reg [31:0] data_2;

	reg [31:0] data_3;

	reg [31:0] data_4;

	reg [31:0] data_5;

	reg [31:0] data_6;

	reg [31:0] data_7;



   reg [31:0]  good_data_0;

   reg [31:0]  good_data_1;

   reg [31:0]  good_data_2;

   reg [31:0]  good_data_3;

   reg [31:0]  good_data_4;

   reg [31:0]  good_data_5;

   reg [31:0]  good_data_6;

   reg [31:0]  good_data_7;

   

	reg [7:0] status;

	begin  		

	 IBUSY = `TRUE;

	$display ($time, "ns, Begining the single Burst Read and Write");

	$fdisplay (pLOG_FILE, $time, "ns, Begining the single Burst Read and Write");





	$display ($time, " ns, Beginning First Write");

	$fdisplay (pLOG_FILE, $time, " ns, Beginning First Write");

	IBUSY = 1'b1;



	ISTATE = "WRITE TEST";

 	start_addr = 32'h00000000;

	 data_0 = 32'h11111111;

	 data_1 = 32'h22222222;

	 data_2 = 32'h33333333;

	 data_3 = 32'h44444444;

	 data_4 = 32'h55555555;

	 data_5 = 32'h66666666;

	 data_6 = 32'h77777777;

	 data_7 = 32'h88888888;

	tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   	while (status [7:6] == 2'b10)

      begin

      $display ("Write issued a retry SDRAM controller not yet ready");

      $fdisplay (pLOG_FILE, "Write issued a retry SDRAM controller not yet ready");

      #5000

      tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

      end





	$display ($time, " ns, First PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, First PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



	$display (" ");

	$fdisplay (pLOG_FILE, " ");

	$display ($time, " ns, Now do another write quickly to make sure the retry is correct");

	$fdisplay (pLOG_FILE, $time, " ns, Now do another write quickly to make sure the retry is correct");

	start_addr = 32'h00000000;

	tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

		$display ($time, " ns, Second PCI write complete");

		$fdisplay (pLOG_FILE, $time, " ns, Second PCI write complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Write issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Write issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR write did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR write did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end

	#250



	$display(" ");

	$fdisplay (pLOG_FILE, " ");

	$display ( $time, " ns, Write to a different address to make sure the address lock function is working");

	$fdisplay (pLOG_FILE, $time, " ns, Write to a different address to make sure the address lock function is working");

	start_addr = 32'h00000800;

	data_0 = 32'hAAAAAAAA;

	data_1 = 32'hBBBBBBBB;

	data_2 = 32'hCCCCCCCC;

	data_3 = 32'hDDDDDDDD;

	data_4 = 32'hEEEEEEEE;

	data_5 = 32'hFFFFFFFF;

	data_6 = 32'h00000000;

	data_7 = 32'h99999999;

	tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Second PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, Second PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end

	$display (" ");

	$fdisplay (pLOG_FILE, " ");







	#250

	$display(" ");

	$fdisplay (pLOG_FILE, " ");

	$display ( $time, " ns, Now try a read from the first address -- It should Retry becuase the data is not ready");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read from the first address -- It should Retry becuase the data is not ready");

	start_addr = 32'h00000000;

	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



	#250

	$display(" ");

	$fdisplay (pLOG_FILE, " ");

	$display ( $time, " ns, Now try a read from the second address -- It should Retry becuase it is attempting to complete the first read");

	$fdisplay (pLOG_FILE,  $time, " ns, But it should not save the new address");

	start_addr = 32'h00000800;

	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



   #250

	$display(" ");

	$fdisplay (pLOG_FILE, " ");

	$display ( $time, " ns, Now try a read from the first address again-- It should Read the data now");

	$fdisplay ( pLOG_FILE, $time, " ns, Now try a read from the first address again-- It should Read the data now");

	start_addr = 32'h00000000;

	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



   good_data_0 = 32'h11111111;

   good_data_1 = 32'h22222222;

   good_data_2 = 32'h33333333;

   good_data_3 = 32'h44444444;

   good_data_4 = 32'h55555555;

   good_data_5 = 32'h66666666;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



   // Now read from the other address that was written

   $display(" ");

	$display ( $time, " ns, Now try a read from the Second address -- It should Retry becuase the data is not ready");

   $fdisplay (pLOG_FILE, " ");

	$fdisplay ( pLOG_FILE, $time, " ns, Now try a read from the Second address -- It should Retry becuase the data is not ready");

	start_addr = 32'h00000800;

	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #500

	$display(" ");

	$display ( $time, " ns, Now try a read from the second address again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read from the second address again-- It should Read the data now");

	start_addr = 32'h00000800;

	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



   good_data_0 = 32'hAAAAAAAA;

   good_data_1 = 32'hBBBBBBBB;

   good_data_2 = 32'hCCCCCCCC;

   good_data_3 = 32'hDDDDDDDD;

   good_data_4 = 32'hEEEEEEEE;

   good_data_5 = 32'hFFFFFFFF;

   good_data_6 = 32'h00000000;

   good_data_7 = 32'h99999999;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end







   $display (" ");

   $display (" Rread from the same address, it should issue a RETRY since the previous read completed");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Rread from the same address, it should issue a RETRY since the previous read completed");

   tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #550



   $display(" ");

	$display ( $time, " ns, Now try a read from the second address again-- It should Read the data now");

   $fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read from the second address again-- It should Read the data now");

	start_addr = 32'h00000800;

	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



   good_data_0 = 32'hAAAAAAAA;

   good_data_1 = 32'hBBBBBBBB;

   good_data_2 = 32'hCCCCCCCC;

   good_data_3 = 32'hDDDDDDDD;

   good_data_4 = 32'hEEEEEEEE;

   good_data_5 = 32'hFFFFFFFF;

   good_data_6 = 32'h00000000;

   good_data_7 = 32'h99999999;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



	IBUSY = `FALSE;

	ISTATE = "IDLE";





   	end

endtask





//--------------------------------------------------------------------------------------------------

task tDO_TEST2;     // Differring Burst Sizes

//--------------------------------------------------------------------------------------------------

   reg [31:0] 	start_addr;

   reg [31:0]	start_data;

   reg [31:0] 	burst_size;

   reg [7:0]   status;



   reg [31:0]  data_0;

   reg [31:0]  data_1;

   reg [31:0]  data_2;

   reg [31:0]  data_3;

   reg [31:0]  data_4;

   reg [31:0]  data_5;

   reg [31:0]  data_6;

   reg [31:0]  data_7;



   reg [31:0]  good_data_0;

   reg [31:0]  good_data_1;

   reg [31:0]  good_data_2;

   reg [31:0]  good_data_3;

   reg [31:0]  good_data_4;

   reg [31:0]  good_data_5;

   reg [31:0]  good_data_6;

   reg [31:0]  good_data_7;



begin

   $display($time, " ns [%s INITIATOR]  Beginning Read / Write Tests with Burst sizes other than 8", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Beginning Read / Write Tests with Burst sizes other than 8", pINST_NAME);

   IBUSY = `TRUE;

   ISTATE = "Other Burst Size";





   #10

   $display (" First a burst of 8 to test the MEMORY_WRITE_N function");

   $fdisplay (pLOG_FILE, " First a burst of 8 to test the MEMORY_WRITE_N function");

   start_addr = 32'h00000020;

   start_data = 32'h12345678;

   burst_size = 32'd8;



	tMEMORY_WRITE_N(start_addr, start_data, burst_size, status);



   if (!status[7])

      begin

      $display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      $fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end

   #50



	$display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

	$fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

   else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #900

	$display(" ");

	$display ( $time, " ns, Now try a read again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again-- It should Read the data now");

	start_addr = 32'h00000020;

	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h12345678;

   good_data_1 = 32'h12345679;

   good_data_2 = 32'h1234567A;

   good_data_3 = 32'h1234567B;

   good_data_4 = 32'h1234567C;

   good_data_5 = 32'h1234567D;

   good_data_6 = 32'h1234567E;

   good_data_7 = 32'h1234567F;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



   $display (" ");

   $display (" Now try a burst size of 9 to see if the target disconnects properly");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now try a burst size of 9 to see if the target disconnects properly");





   start_addr = 32'h00000120;

   start_data = 32'h87654321;

   burst_size = 32'd9;



	tMEMORY_WRITE_N(start_addr, start_data, burst_size, status);



   if (!status[7])

      begin

      $display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      $fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end

   #50



   $display (" ");

   $fdisplay (pLOG_FILE, " ");

   $display (" Now try a read of 10 items from the previous address -- The first attempt should retry");

   $fdisplay (pLOG_FILE, " Now try a read of 10 items from the previous address -- The first attempt should retry");



   burst_size = 32'd10;

   tMEMORY_READ_N(start_addr, burst_size, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1)

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #900

	$display(" ");

   $display(" Now try the read again, the data should be ready");

   tMEMORY_READ_N(start_addr, burst_size, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h87654321;

   good_data_1 = 32'h87654322;

   good_data_2 = 32'h87654323;

   good_data_3 = 32'h87654324;

   good_data_4 = 32'h87654325;

   good_data_5 = 32'h87654326;

   good_data_6 = 32'h87654327;

   good_data_7 = 32'h87654328;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



   $display (" ");





   $display (" Now do a few writes of less than 8 to make sure that the data padding is acting correctly ");

   $display (" First write 8 words to have the memory filled with known values");

   $fdisplay (pLOG_FILE, " Now do a few writes of less than 8 to make sure that the data padding is acting correctly ");

   $fdisplay (pLOG_FILE, " First write 8 words to have the memory filled with known values");

   start_addr = 32'h00000060;

   data_0 = 32'hC001CAFE;

   data_1 = 32'hC001CAFE;

   data_2 = 32'hC001CAFE;

   data_3 = 32'hC001CAFE;

   data_4 = 32'hC001CAFE;

   data_5 = 32'hC001CAFE;

   data_6 = 32'hC001CAFE;

   data_7 = 32'hC001CAFE;



	tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, First PCI write complete");

   $fdisplay (pLOG_FILE, $time, " ns, First PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end





   #550



   $display (" ");

   $display ("Now a burst of 7 data items");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, "Now a burst of 7 data items");

   start_data = 32'h22222222;

   burst_size = 32'd7;

   tMEMORY_WRITE_N (start_addr, start_data, burst_size, status);

   if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #1000

	$display(" ");

	$display ( $time, " ns, Now try a read again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again-- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h22222222;

   good_data_1 = 32'h22222223;

   good_data_2 = 32'h22222224;

   good_data_3 = 32'h22222225;

   good_data_4 = 32'h22222226;

   good_data_5 = 32'h22222227;

   good_data_6 = 32'h22222228;

   good_data_7 = 32'hC001CAFE;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end





      #350



   $display (" ");

   $display ("Now a burst of 6 data items");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, "Now a burst of 6 data items");

   start_data = 32'h66666666;

   burst_size = 32'd6;

   tMEMORY_WRITE_N (start_addr, start_data, burst_size, status);



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again-- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h66666666;

   good_data_1 = 32'h66666667;

   good_data_2 = 32'h66666668;

   good_data_3 = 32'h66666669;

   good_data_4 = 32'h6666666A;

   good_data_5 = 32'h6666666B;

   good_data_6 = 32'h22222228;

   good_data_7 = 32'hC001CAFE;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



   $display (" ");

   $display ("Now a burst of 5 data items");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, "Now a burst of 5 data items");

   start_data = 32'h55555555;

   burst_size = 32'd5;

   tMEMORY_WRITE_N (start_addr, start_data, burst_size, status);



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again-- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h55555555;

   good_data_1 = 32'h55555556;

   good_data_2 = 32'h55555557;

   good_data_3 = 32'h55555558;

   good_data_4 = 32'h55555559;

   good_data_5 = 32'h6666666B;

   good_data_6 = 32'h22222228;

   good_data_7 = 32'hC001CAFE;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



   $display (" ");

   $display ("Now a burst of 4 data items");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, "Now a burst of 4 data items");

   start_data = 32'h44444444;

   burst_size = 32'd4;

   tMEMORY_WRITE_N (start_addr, start_data, burst_size, status);



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again-- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h44444444;

   good_data_1 = 32'h44444445;

   good_data_2 = 32'h44444446;

   good_data_3 = 32'h44444447;

   good_data_4 = 32'h55555559;

   good_data_5 = 32'h6666666B;

   good_data_6 = 32'h22222228;

   good_data_7 = 32'hC001CAFE;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



	   $display (" ");

   $display ("Now a burst of 3 data items");

	   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, "Now a burst of 3 data items");

   start_data = 32'h33333333;

   burst_size = 32'd3;

   tMEMORY_WRITE_N (start_addr, start_data, burst_size, status);



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again-- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h33333333;

   good_data_1 = 32'h33333334;

   good_data_2 = 32'h33333335;

   good_data_3 = 32'h44444447;

   good_data_4 = 32'h55555559;

   good_data_5 = 32'h6666666B;

   good_data_6 = 32'h22222228;

   good_data_7 = 32'hC001CAFE;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



	   $display (" ");

   $display ("Now a burst of 2 data items");

	   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, "Now a burst of 2 data items");

   start_data = 32'h22222222;

   burst_size = 32'd2;

   tMEMORY_WRITE_N (start_addr, start_data, burst_size, status);



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again-- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h22222222;

   good_data_1 = 32'h22222223;

   good_data_2 = 32'h33333335;

   good_data_3 = 32'h44444447;

   good_data_4 = 32'h55555559;

   good_data_5 = 32'h6666666B;

   good_data_6 = 32'h22222228;

   good_data_7 = 32'hC001CAFE;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end



	   $display (" ");

   $display ("Now a burst of a single data item");

	   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, "Now a burst of a single data item");

   start_data = 32'h11111111;

   burst_size = 32'd1;

   tMEMORY_WRITE_N (start_addr, start_data, burst_size, status);



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again-- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h22222223;

   good_data_2 = 32'h33333335;

   good_data_3 = 32'h44444447;

   good_data_4 = 32'h55555559;

   good_data_5 = 32'h6666666B;

   good_data_6 = 32'h22222228;

   good_data_7 = 32'hC001CAFE;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end





   $display($time, " ns [%s INITIATOR]  Ending other burst size Tests", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Ending other burst size Tests", pINST_NAME);

   IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask





//--------------------------------------------------------------------------------------------------

task tDO_TEST3;     // Initiator Wait States

//--------------------------------------------------------------------------------------------------



   reg [31:0] 	start_addr;

   reg [31:0]	start_data;

   reg [31:0] 	burst_size;

   reg [7:0]   status;



   reg [31:0]  data_0;

   reg [31:0]  data_1;

   reg [31:0]  data_2;

   reg [31:0]  data_3;

   reg [31:0]  data_4;

   reg [31:0]  data_5;

   reg [31:0]  data_6;

   reg [31:0]  data_7;



   reg [31:0]  good_data_0;

   reg [31:0]  good_data_1;

   reg [31:0]  good_data_2;

   reg [31:0]  good_data_3;

   reg [31:0]  good_data_4;

   reg [31:0]  good_data_5;

   reg [31:0]  good_data_6;

   reg [31:0]  good_data_7;





   begin

   $display($time, " ns [%s INITIATOR]  Begin Test3, Wait states....", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Begin Test3, Wait states....", pINST_NAME);

   IBUSY = `TRUE;

   ISTATE = "Wait state tests";



   $display (" ");

   $display ("Write data to the FIFO");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, "Write data to the FIFO");

   start_addr = 32'h00000000;

	data_0 = 32'h11111111;

	data_1 = 32'h22222222;

	data_2 = 32'h33333333;

	data_3 = 32'h44444444;

	data_4 = 32'h55555555;

	data_5 = 32'h66666666;

	data_6 = 32'h77777777;

	data_7 = 32'h88888888;

	tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, First PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, First PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again with wait states-- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again with wait states-- It should Read the data now");



	tMEMORY_READ_8_WAIT(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h22222222;

   good_data_2 = 32'h33333333;

   good_data_3 = 32'h44444444;

   good_data_4 = 32'h55555555;

   good_data_5 = 32'h66666666;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end







   $display($time, " ns [%s INITIATOR]  Test3  (Self ID) End", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Test3  (Self ID) End", pINST_NAME);

   IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask





//--------------------------------------------------------------------------------------------------

task tDO_TEST4;      // Byte Enable Test



//--------------------------------------------------------------------------------------------------

   reg [31:0] 	start_addr;

   reg [31:0]	start_data;

   reg [31:0] 	burst_size;

   reg [7:0]   status;



   reg [31:0]  data_0;

   reg [31:0]  data_1;

   reg [31:0]  data_2;

   reg [31:0]  data_3;

   reg [31:0]  data_4;

   reg [31:0]  data_5;

   reg [31:0]  data_6;

   reg [31:0]  data_7;



   reg [31:0]  good_data_0;

   reg [31:0]  good_data_1;

   reg [31:0]  good_data_2;

   reg [31:0]  good_data_3;

   reg [31:0]  good_data_4;

   reg [31:0]  good_data_5;

   reg [31:0]  good_data_6;

   reg [31:0]  good_data_7;



   reg [3:0]   cbe_0;

   reg [3:0]   cbe_1;

   reg [3:0]   cbe_2;

   reg [3:0]   cbe_3;

   reg [3:0]   cbe_4;

   reg [3:0]   cbe_5;

   reg [3:0]   cbe_6;

   reg [3:0]   cbe_7;





	begin

   $display($time, " ns [%s INITIATOR]  Begin Test4 (Byte Enable Test)", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Begin Test4 (Byte Enable Test)", pINST_NAME);

   IBUSY = `TRUE;

   ISTATE = "BYTE ENABLE";



   $display ($time, " ns, FIrst write data so there is a known 'background'");

   $fdisplay (pLOG_FILE, $time, " ns, FIrst write data so there is a known 'background'");

   data_0 = 32'h12345678;

   data_1 = 32'h12345678;

   data_2 = 32'h12345678;

   data_3 = 32'h12345678;

   data_4 = 32'h12345678;

   data_5 = 32'h12345678;

   data_6 = 32'h12345678;

   data_7 = 32'h12345678;

   start_addr =32'h00000040;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, First PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, First PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h12345678;

   good_data_1 = 32'h12345678;

   good_data_2 = 32'h12345678;

   good_data_3 = 32'h12345678;

   good_data_4 = 32'h12345678;

   good_data_5 = 32'h12345678;

   good_data_6 = 32'h12345678;

   good_data_7 = 32'h12345678;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end





      $display ($time, " ns, Now write data with byte enables set");

      $fdisplay (pLOG_FILE, $time, " ns, Now write data with byte enables set");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   cbe_0 = 4'b1000;

   cbe_1 = 4'b0100;

   cbe_2 = 4'b0010;

   cbe_3 = 4'b0001;

   cbe_4 = 4'b1010;

   cbe_5 = 4'b1111;

   cbe_6 = 4'b0000;

   cbe_7 = 4'b0101;

   start_addr =32'h00000040;



   tMEMORY_WRITE_8_CBE (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, cbe_0, cbe_1, cbe_2,

								cbe_3, cbe_4, cbe_5, cbe_6, cbe_7, status);

	$display ($time, " ns, First PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, First PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #350



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h12111111;

   good_data_1 = 32'h22342222;

   good_data_2 = 32'h33335633;

   good_data_3 = 32'h44444478;

   good_data_4 = 32'h12555655;

   good_data_5 = 32'h12345678;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88348878;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end







   $display($time, " ns [%s INITIATOR]  Test4 (Byte Enable End", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Test4 (Byte Enable End", pINST_NAME);

   IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask







//-------------------------------------------------------------------------------------------------

task tDO_TEST5;		 // Writes to other offsets



//-----------------------------------------------------------------------------------------------

   reg [31:0] 	start_addr;

   reg [31:0]	start_data;

   reg [31:0] 	burst_size;

   reg [7:0]   status;



   reg [31:0]  data_0;

   reg [31:0]  data_1;

   reg [31:0]  data_2;

   reg [31:0]  data_3;

   reg [31:0]  data_4;

   reg [31:0]  data_5;

   reg [31:0]  data_6;

   reg [31:0]  data_7;



   reg [31:0]  good_data_0;

   reg [31:0]  good_data_1;

   reg [31:0]  good_data_2;

   reg [31:0]  good_data_3;

   reg [31:0]  good_data_4;

   reg [31:0]  good_data_5;

   reg [31:0]  good_data_6;

   reg [31:0]  good_data_7;



   reg [3:0]   cbe_0;

   reg [3:0]   cbe_1;

   reg [3:0]   cbe_2;

   reg [3:0]   cbe_3;

   reg [3:0]   cbe_4;

   reg [3:0]   cbe_5;

   reg [3:0]   cbe_6;

   reg [3:0]   cbe_7;









   begin

   $display($time, " ns [%s INITIATOR]  Begin Test 5....", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Begin Test 5....", pINST_NAME);

   IBUSY = `TRUE;

   ISTATE = "TEST5";



   $display ($time, " ns, First write with no offset");

   $fdisplay (pLOG_FILE, $time, " ns, First write with no offset");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   start_addr =32'h00000040;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, First PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, First PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #850



   $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   start_addr =32'h00000040;

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h22222222;

   good_data_2 = 32'h33333333;

   good_data_3 = 32'h44444444;

   good_data_4 = 32'h55555555;

   good_data_5 = 32'h66666666;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end















   $display ($time, " ns, Second write with  offset of 4");

   $fdisplay (pLOG_FILE, $time, " ns, Second write with  offset of 4");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   start_addr =32'h00000044;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Second PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, Second PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #850



  $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

  $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   start_addr =32'h00000040;

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h11111111;

   good_data_2 = 32'h33333333;

   good_data_3 = 32'h44444444;

   good_data_4 = 32'h55555555;

   good_data_5 = 32'h66666666;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end













   $display ($time, " ns, Third write with  offset of 8");

   $fdisplay (pLOG_FILE, $time, " ns, Third write with  offset of 8");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   start_addr =32'h00000048;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Third PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, Third PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #850



  $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

  $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   start_addr =32'h00000040;

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h11111111;

   good_data_2 = 32'h11111111;

   good_data_3 = 32'h44444444;

   good_data_4 = 32'h55555555;

   good_data_5 = 32'h66666666;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end











   $display ($time, " ns, Fourth write with  offset of C");

   $fdisplay (pLOG_FILE, $time, " ns, Fourth write with  offset of C");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   start_addr =32'h0000004C;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Fourth PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, Fourth PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #850



  $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

  $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   start_addr =32'h00000040;

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h11111111;

   good_data_2 = 32'h11111111;

   good_data_3 = 32'h11111111;

   good_data_4 = 32'h55555555;

   good_data_5 = 32'h66666666;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end











   $display ($time, " ns, Fifth write with  offset of 10");

   $fdisplay (pLOG_FILE, $time, " ns, Fifth write with  offset of 10");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   start_addr =32'h00000050;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Fifth PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, Fifth PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #850



  $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

  $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   start_addr =32'h00000040;

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h11111111;

   good_data_2 = 32'h11111111;

   good_data_3 = 32'h11111111;

   good_data_4 = 32'h11111111;

   good_data_5 = 32'h66666666;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end











   $display ($time, " ns, Sixth  write with  offset of 14");

   $fdisplay (pLOG_FILE, $time, " ns, Sixth  write with  offset of 14");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   start_addr =32'h00000054;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Sixth PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, Sixth PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #850



     $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

     $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   start_addr =32'h00000040;

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h11111111;

   good_data_2 = 32'h11111111;

   good_data_3 = 32'h11111111;

   good_data_4 = 32'h11111111;

   good_data_5 = 32'h11111111;

   good_data_6 = 32'h77777777;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end













   $display ($time, " ns, Seventh write with offset of 18");

   $fdisplay (pLOG_FILE, $time, " ns, Seventh write with offset of 18");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   start_addr =32'h00000058;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Seventh PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, Seventh PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #850



  $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

  $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   start_addr =32'h00000040;

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h11111111;

   good_data_2 = 32'h11111111;

   good_data_3 = 32'h11111111;

   good_data_4 = 32'h11111111;

   good_data_5 = 32'h11111111;

   good_data_6 = 32'h11111111;

   good_data_7 = 32'h88888888;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end











   $display ($time, " ns, Eighth write with  offset of 1C");

   $fdisplay (pLOG_FILE, $time, " ns, Eighth write with  offset of 1C");

   data_0 = 32'h11111111;

   data_1 = 32'h22222222;

   data_2 = 32'h33333333;

   data_3 = 32'h44444444;

   data_4 = 32'h55555555;

   data_5 = 32'h66666666;

   data_6 = 32'h77777777;

   data_7 = 32'h88888888;

   start_addr =32'h0000005C;



   tMEMORY_WRITE_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Eighth PCI write complete");

	$fdisplay (pLOG_FILE, $time, " ns, Eighth PCI write complete");

	if (!status [7]) begin

		$display ("Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

		$fdisplay (pLOG_FILE, "Write complete with status = %b and %d data items written", status[7:6], status[5:0]);

      end

	else

		begin

		$display ("ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$fdisplay (pLOG_FILE, "ERROR write did not complete as expected:: Status = %b ", status[7:6]);

		$stop(2);

		end



   #850



  $display (" ");

   $display (" Now attempt to read it back (It will retry on this attempt)");

  $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " Now attempt to read it back (It will retry on this attempt)");

   start_addr =32'h00000040;

   tMEMORY_READ_8 (start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

   $display ($time, " ns, Read attempt complete");

   $fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (status [7:6] == 2'b10 && status [5:0] == 1) begin

		$display ("Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "Read issued RETRY as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      end

	else

		begin

		$display ("ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not RETRY as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end



      #950

	$display(" ");

	$display ( $time, " ns, Now try a read again -- It should Read the data now");

	$fdisplay (pLOG_FILE, " ");

	$fdisplay (pLOG_FILE,  $time, " ns, Now try a read again -- It should Read the data now");



	tMEMORY_READ_8(start_addr, data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);

	$display ($time, " ns, Read attempt complete");

	$fdisplay (pLOG_FILE, $time, " ns, Read attempt complete");

	if (!status[7])

      begin

		$display ("Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $display ("DATA [0] = %hH", data_0);

      $display ("DATA [1] = %hH", data_1);

      $display ("DATA [2] = %hH", data_2);

      $display ("DATA [3] = %hH", data_3);

      $display ("DATA [4] = %hH", data_4);

      $display ("DATA [5] = %hH", data_5);

      $display ("DATA [6] = %hH", data_6);

      $display ("DATA [7] = %hH", data_7);

		$fdisplay (pLOG_FILE, "Read complete as expected -- Status = %b Number of Transactions = %d", status [7:6], status [5:0]);

      $fdisplay (pLOG_FILE, "DATA [0] = %hH", data_0);

      $fdisplay (pLOG_FILE, "DATA [1] = %hH", data_1);

      $fdisplay (pLOG_FILE, "DATA [2] = %hH", data_2);

      $fdisplay (pLOG_FILE, "DATA [3] = %hH", data_3);

      $fdisplay (pLOG_FILE, "DATA [4] = %hH", data_4);

      $fdisplay (pLOG_FILE, "DATA [5] = %hH", data_5);

      $fdisplay (pLOG_FILE, "DATA [6] = %hH", data_6);

      $fdisplay (pLOG_FILE, "DATA [7] = %hH", data_7);

      end

	else

		begin

		$display ("ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$fdisplay (pLOG_FILE, "ERROR read did not complete as expected:: Status = %b, Number of Transactions = %d", status[7:6], status [5:0]);

		$stop(2);

		end





   good_data_0 = 32'h11111111;

   good_data_1 = 32'h11111111;

   good_data_2 = 32'h11111111;

   good_data_3 = 32'h11111111;

   good_data_4 = 32'h11111111;

   good_data_5 = 32'h11111111;

   good_data_6 = 32'h11111111;

   good_data_7 = 32'h11111111;



   tCHECK_DATA_8 (good_data_0, good_data_1, good_data_2, good_data_3, good_data_4, good_data_5, good_data_6, good_data_7,

                  data_0, data_1, data_2, data_3, data_4, data_5, data_6, data_7, status);



   if (status == 8'hFF)

      begin

      $display (" Data is correct -- Status = %hH", status);

      $fdisplay (pLOG_FILE, " Data is correct -- Status = %hH", status);

      end

   else

   	begin

      $display ("Data contains Errors -- Status = %hH", status);

      $fdisplay (pLOG_FILE, "Data contains Errors -- Status = %hH", status);

      $stop(2);

      end







   $display($time, " ns [%s INITIATOR] Test 5 End", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR] Test 5 End", pINST_NAME);



   IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask



//--------------------------------------------------------------------------------------------------

task tDO_TEST6;   // Register Test



//--------------------------------------------------------------------------------------------------

   reg [31:0] 	data;

   reg [31:0] 	start_addr;

   reg [7:0]   status;



	begin

   $display($time, " ns [%s INITIATOR]  Begin Test 6....", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Begin Test 6....", pINST_NAME);

   IBUSY = `TRUE;

   ISTATE = "TEST6";

   $display ($time, " ns");

   $display (" ");

   $display (" ");

   $fdisplay (pLOG_FILE, $time, " ns");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, " ");



/*   $display ("Testing read from register 0");

   start_addr = 32'h10000000;

   tMEMORY_READ_1(start_addr, data, status);



   $display ("Memory read = %h", data);



   $display ("Testing read from register 1");

   start_addr = 32'h10000004;

   tMEMORY_READ_1(start_addr, data, status);



   $display ("Memory read = %h", data);  */



   $display ("Testing read from register 8");

   $fdisplay (pLOG_FILE, "Testing read from register 8");

   start_addr = 32'h10000020;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[2:0] != 3'b0) $stop(2);



   $display (" ");

   $display ($time, " ns -- Now write to the registers");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, $time, " ns -- Now write to the registers");

   start_addr = 32'h10000000;

   data = 32'h55555555;

   tMEMORY_WRITE_1 (start_addr, data, status);

   #250



   start_addr = 32'h10000004;

   data = 32'h33331234;

   tMEMORY_WRITE_1 (start_addr, data, status);

   #250



   start_addr = 32'h10000008;

   data = 32'h89ABCDEF;

   tMEMORY_WRITE_1 (start_addr, data, status);

   #250



   start_addr = 32'h1000000C;

   data = 32'h01234567;

   tMEMORY_WRITE_1 (start_addr, data, status);

   #250



   start_addr = 32'h1000001C;

   data = 32'h00000007;

   tMEMORY_WRITE_1 (start_addr, data, status);

   #250



   $display (" ");

   $display ($time, " ns -- Now Read Back");

   $fdisplay (pLOG_FILE, " ");

   $fdisplay (pLOG_FILE, $time, " ns -- Now Read Back");



   $display ("Testing read from register 0");

   $fdisplay (pLOG_FILE, "Testing read from register 0");

   start_addr = 32'h10000000;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (!data[0]) $stop(2);



   start_addr = 32'h10000000;

   data = 32'hAAAAAAAA;

   tMEMORY_WRITE_1 (start_addr, data, status);

   #250



   $display ("Testing read from register 0");

   $fdisplay (pLOG_FILE, "Testing read from register 0");

   start_addr = 32'h10000000;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[0]) $stop(2);



   $display ("Testing read from register 9");

   $fdisplay (pLOG_FILE, "Testing read from register 9");

   start_addr = 32'h10000024;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[0]) $stop(2);



   start_addr = 32'h10000000;

   data = 32'h55555555;

   tMEMORY_WRITE_1 (start_addr, data, status);

   #250



   $display ("Testing read from register 1");

   $fdisplay (pLOG_FILE, "Testing read from register 1");

   start_addr = 32'h10000004;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[7:0] != 8'h34) $stop(2);



   $display ("Testing read from register 6");

   $fdisplay (pLOG_FILE, "Testing read from register 6");

   start_addr = 32'h10000018;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[7:0] != 8'h34) $stop(2);



   start_addr = 32'h10000004;

   data = 32'h33331282;

   tMEMORY_WRITE_1 (start_addr, data, status);

   #250



   $display ("Testing read from register 1");

   $fdisplay (pLOG_FILE, "Testing read from register 1");

   start_addr = 32'h10000004;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[7:0] != 8'h82) $stop(2);



   $display ("Testing read from register 6");

   $fdisplay (pLOG_FILE, "Testing read from register 6");

   start_addr = 32'h10000018;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[7:0] != 8'h82) $stop(2);



   $display ("Testing read from register 2");

   $fdisplay (pLOG_FILE, "Testing read from register 2");

   start_addr = 32'h10000008;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[31:0] != 32'h89ABCDEF) $stop(2);



   $display ("Testing read from register 3");

   $fdisplay (pLOG_FILE, "Testing read from register 3");

   start_addr = 32'h1000000C;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[23:0] != 24'h234567) $stop(2);



   $display ("Testing read from register 4");

   $fdisplay (pLOG_FILE, "Testing read from register 4");

   start_addr = 32'h10000010;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[31:0] != 32'h89ABCDEF) $stop(2);



   $display ("Testing read from register 5");

   $fdisplay (pLOG_FILE, "Testing read from register 5");

   start_addr = 32'h10000014;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[23:0] != 24'h234567) $stop(2);



   $display ("Testing read from register 7");

   $fdisplay (pLOG_FILE, "Testing read from register 7");

   start_addr = 32'h1000001C;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[2:0] != 3'h7) $stop(2);



   $display ("Testing read from register 8");

   $fdisplay (pLOG_FILE, "Testing read from register 8");

   start_addr = 32'h10000020;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[2:0] != 3'h7) $stop(2);



   $display ("Testing read from register 9");

   $fdisplay (pLOG_FILE, "Testing read from register 9");

   start_addr = 32'h10000024;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (!data[0]) $stop(2);



   $display ("Testing read from register 8");

   $fdisplay (pLOG_FILE, "Testing read from register 8");

   start_addr = 32'h10000020;

   tMEMORY_READ_1(start_addr, data, status);

   $display ("Memory read = %h", data);

   $fdisplay (pLOG_FILE, "Memory read = %h", data);

   if (data[2:0] != 3'h0) $stop(2);





   #500

   $display($time, " ns [%s INITIATOR] Test 6 End", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR] Test 6 End", pINST_NAME);

   IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask



//-------------------------------------------------------------------------------------------------

task tDO_TEST7;    // Test 7   UART TEST



//-----------------------------------------------------------------------------------------------

   reg [31:0] 	data;

   reg [31:0] 	tx_data;

   reg [31:0] 	start_addr;

   reg [7:0]   status;

   reg [15:0]  baud_vec [9:0];

   integer     i;

   integer     baud_rate;



   begin

   $display($time, " ns [%s INITIATOR]  Begin Test 7....", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR]  Begin Test 7....", pINST_NAME);

   IBUSY = `TRUE;

   ISTATE = "TEST7";



   //-------- initialization -----------

      // 1 disable all interrupt enable bits

         start_addr = 32'h1000001C;

         data = 32'h0;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250



      // 2 read int_status register to clear INTR_A and check it

         start_addr = 32'h10000020;

         tMEMORY_READ_1(start_addr, data, status);

         #300

         if (!INTR_A) begin

            $display ("******ERROR******: INTR_A can't be cleared at step 1.2!!!!");

            $fdisplay (pLOG_FILE, "******ERROR******: INTR_A can't be cleared at step 1.2!!!!");

            $stop(2);

            end



      // 3 initialize BAUD_RATE table

         baud_vec[0] = 16'hD9;

         baud_vec[1] = 16'h6C;

         baud_vec[2] = 16'h48;

         baud_vec[3] = 16'h36;

         baud_vec[4] = 16'h2B;

         baud_vec[5] = 16'h24;

         baud_vec[6] = 16'h1F;

         baud_vec[7] = 16'h1B;

         baud_vec[8] = 16'h18;

         baud_vec[9] = 16'h12;



      // 4 uart status check (rx_rdy = 0, tx_rdy = 1)

         start_addr = 32'h1000003C;

         tMEMORY_READ_1(start_addr, data, status);

         if (data != 32'h00050001) begin

            $display ("Actual UART STATUS register = %h; Expected = 32'h00050001", data);

            $fdisplay (pLOG_FILE, "Actual UART STATUS register = %h; Expected = 32'h00050001", data);

            $stop(2);

            end



// test loop

   tx_data = 32'hFFFFFFFF;

   for (i = 10; i > 8; i = i - 1) begin

   //-------- transmit cycle -----------

      // 1 set Baud Rate

         start_addr = 32'h10000038;

         data[31:16] = 16'h3;

         data[15:0] = baud_vec[i-1];

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250



      // 2 write 4 bytes to txfifo

         start_addr = 32'h10000030;

         data = tx_data;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250

         start_addr = 32'h10000030;

         data = tx_data - 1;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250

         start_addr = 32'h10000030;

         data = tx_data - 2;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250

         start_addr = 32'h10000030;

         data = tx_data - 3;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250



      // 3 enable tx_intr

         start_addr = 32'h1000001C;

         data = 32'h11;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250



      // 4 wait for INTR_A = 0

         wait (INTR_A == 1'b0);

         $display ("***************** A INTERRUPT OCCURS ****************");

         $fdisplay (pLOG_FILE, "***************** A INTERRUPT OCCURS ****************");



      // 5 disable tx_intr

         start_addr = 32'h1000001C;

         data = 32'h0;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250



      // 6 read int_status register to clear INTR_A

         start_addr = 32'h10000020;

         tMEMORY_READ_1(start_addr, data, status);

         #300

         if (!INTR_A) begin

            $display ("******ERROR******: INTR_A can't be cleared at step 2.6!!!!");

            $fdisplay (pLOG_FILE, "******ERROR******: INTR_A can't be cleared at step 2.6!!!!");

            $stop(2);

            end





   //-------- receive cycle -----------

      // 1 enable rx_intr

         start_addr = 32'h1000001C;

         data = 32'h9;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250



      // 2 wait for INTR_A

         wait (INTR_A == 1'b0);

         $display ("***************** A INTERRUPT OCCURS ****************");

         $fdisplay (pLOG_FILE, "***************** A INTERRUPT OCCURS ****************");



      // 3 disable rx_intr

         start_addr = 32'h1000001C;

         data = 32'h0;

         tMEMORY_WRITE_1 (start_addr, data, status);

         #250



      // 4 read int_status_register to clear INTR_A

         start_addr = 32'h10000020;

         tMEMORY_READ_1(start_addr, data, status);

         #300

         if (!INTR_A) begin

            $display ("******ERROR******: INTR_A can't be cleared at step 2.6!!!!");

            $fdisplay (pLOG_FILE, "******ERROR******: INTR_A can't be cleared at step 2.6!!!!");

            $stop(2);

            end



      // 5 read data from rxfifo and verify them

         start_addr = 32'h10000034;

         tMEMORY_READ_1 (start_addr, data, status);

         if (data[7:0] != tx_data[7:0]) begin

            $display (" ****************Error Received Data %d*******", ((10-i)*4 + 1));

            $display (" ***** Actual Data = %h; Expected = %h", data[7:0], tx_data[7:0]);

            $fdisplay (pLOG_FILE, " ****************Error Received Data %d*******", ((10-i)*4 + 1));

            $fdisplay (pLOG_FILE, " ***** Actual Data = %h; Expected = %h", data[7:0], tx_data[7:0]);

            $stop(2);

            end

         start_addr = 32'h10000034;

         tMEMORY_READ_1 (start_addr, data, status);

         if (data[7:0] != tx_data[7:0] - 8'h1) begin

            $display (" ****************Error Received Data %d*******", ((10-i)*4 + 2));

            $display (" ***** Actual Data = %h; Expected = %h", data[7:0], (tx_data[7:0] - 8'h1));

            $fdisplay (pLOG_FILE, " ****************Error Received Data %d*******", ((10-i)*4 + 2));

            $fdisplay (pLOG_FILE, " ***** Actual Data = %h; Expected = %h", data[7:0], (tx_data[7:0] - 8'h1));

            $stop(2);

            end

         start_addr = 32'h10000034;

         tMEMORY_READ_1 (start_addr, data, status);

         if (data[7:0] != tx_data[7:0] - 8'h2) begin

            $display (" ****************Error Received Data %d*******", ((10-i)*4 + 3));

            $display (" ***** Actual Data = %h; Expected = %h", data[7:0], (tx_data[7:0] - 8'h2));

            $fdisplay (pLOG_FILE, " ****************Error Received Data %d*******", ((10-i)*4 + 3));

            $fdisplay (pLOG_FILE, " ***** Actual Data = %h; Expected = %h", data[7:0], (tx_data[7:0] - 8'h2));

            $stop(2);

            end

         start_addr = 32'h10000034;

         tMEMORY_READ_1 (start_addr, data, status);

         if (data[7:0] != tx_data[7:0] - 8'h3) begin

            $display (" ****************Error Received Data %d*******", ((10-i)*4 + 4));

            $display (" ***** Actual Data = %h; Expected = %h", data[7:0], (tx_data[7:0] - 8'h3));

            $fdisplay (pLOG_FILE, " ****************Error Received Data %d*******", ((10-i)*4 + 4));

            $fdisplay (pLOG_FILE, " ***** Actual Data = %h; Expected = %h", data[7:0], (tx_data[7:0] - 8'h3));

            $stop(2);

            end







      tx_data = tx_data - 4;

      if (i == 10) baud_rate = 9600 * 12;

      else    baud_rate = 9600 * i;

      $display (" ");

      $display (" ");

      $display ("*********end of baud rate %d test***********", baud_rate);

      $fdisplay (pLOG_FILE, "*********end of baud rate %d test***********", baud_rate);

      $display (" ");

      $display (" ");

      $display (" ");

      $display (" ");

   end



   $display($time, " ns [%s INITIATOR] Test 7 End", pINST_NAME);

   $fdisplay (pLOG_FILE, $time, " ns [%s INITIATOR] Test 7 End", pINST_NAME);

   IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask



//-------------------------------------------------------------------------------------------------

task tDO_TEST8;    // Test 8



//-----------------------------------------------------------------------------------------------

   begin

   $display($time, " ns [%s INITIATOR]  Begin Test 8....", pINST_NAME);

   IBUSY = `TRUE;

   ISTATE = "TEST8";





   $display($time, " ns [%s INITIATOR] Test 8 End", pINST_NAME);

   IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask



//-------------------------------------------------------------------------------------------------

task tDO_TEST9;    // Test 9



//-----------------------------------------------------------------------------------------------

   begin

   $display($time, " ns [%s INITIATOR]  Begin Test 9....", pINST_NAME);

   IBUSY = `TRUE;

   ISTATE = "TEST9";





   $display($time, " ns [%s INITIATOR] Test 9 End", pINST_NAME);

   IBUSY = `FALSE;

   ISTATE = "IDLE";

   end

endtask











//-----------------------------------------------

//  Add in Initiator lowlevel PCI bus tasks

//-----------------------------------------------

`include "PCI_device_initiator_bus.v"

`include "PCI_device_initiator_sdram.v"







