/*----------------------------------------------------------------------------------------------------

--

-- File name	 : userapp.V

--					 :

-- Title			 : USERAPP

--					 :

-- Purpose		 : This is the top level of the SDRAM interface for The Spartan 2 Reference Board

--					 :

-- IncludeFiles :

--					 :

-- Created On	 : August 23, 2001

--					 :

-- Comments		 :

--					 :

--					 :

--					 : For the Spartan 2 Reference Board

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

--					 :

-- Conventions  : - Port names are upper case.

--					 : - Internal signals and variables are lower case.

--					 : - Active low signals are identified with '_n' or '_N'.

--					 : - constants begin with lower case 'c', rest upper case

--					 : - states begin with lower case 's', rest upper case

--					 : - aliases begin with lower case 'a', rest upper case

--					 :

--					 : Tab setting = 3

--					 : Right Margin = 100

--					 :

-- Limitations  : For use in Xilinx FPGA

--					 :

-- Developers	 : Duncan Allen

----------------------------------------------------------------------------------------------------

-- Revision History :

----------------------------------------------------------------------------------------------------

--	  Version No 	| Author	 	| Changes Made:							| Mod. Date:

--		v1.0			| da			| First write								| 23/8/01

----------------------------------------------------------------------------------------------------*/



// This is a simple synchronous FIFO.

// It assumes that each burst of data will be 8 units long and that

// each burst will be emptied before a new one is loaded





// This module contains the PCI -- SDRAM controller



`timescale 1 ns/1 ps



module userapp (

//    FRAMEQ_N,

//    TRDYQ_N,

//    IRDYQ_N,

//    STOPQ_N,

//    DEVSELQ_N,

    ADDR,

    ADIO,

 //   CFG_VLD,

 //   CFG_HIT,

    C_TERM,

    C_READY,

//    ADDR_VLD,

    BASE_HIT,

    S_TERM,

    S_READY,

    S_ABORT,

    S_WRDN,

    S_SRC_EN,

    S_DATA_VLD,

    S_CBE,

//    PCI_CMD,

    REQUEST,

    REQUESTHOLD,

    COMPLETE,

    M_WRDN,

    M_READY,

//    M_SRC_EN,

//    M_DATA_VLD,

    M_CBE,

//  TIME_OUT,

    CFG_SELF,

//    M_DATA,

//    DR_BUS,

//    I_IDLE,

//    M_ADDR_N,

//    IDLE,

//    B_BUSY,

    S_DATA,

//    BACKOFF,

    INTR_N,

//    PERRQ_N,

//    SERRQ_N,

    KEEPOUT,

//    CSR,

    SUB_DATA,

//    CFG,

    RST,

	CLK,



   // The SDRAM Interface

    SD_ADDR,

	SD_BS,

//	SD_CLK,

	SD_CKE,

	SD_CS_N,

	SD_RAS_N,

	SD_CAS_N,

	SD_WE_N,

	SD_DQM,

	SD_DATA,

   // The seven segment LED signals

   DISPLAY1A,

   DISPLAY1B,

   DISPLAY1C,

   DISPLAY1D,

   DISPLAY1E,

   DISPLAY1F,

   DISPLAY1G,

   DISPLAY2A,

   DISPLAY2B,

   DISPLAY2C,

   DISPLAY2D,

   DISPLAY2E,

   DISPLAY2F,

   DISPLAY2G,  

   LED_USER,

   DIP,      

   PUSH_USER1,  

   SIN,

   SOUT,

   SIG_IN,

   SIG_OUT

	);





// synthesis syn_hier = "hard"

// synthesis syn_noclockbuf = 1

// synthesis syn_edif_bit_format = "%u<%i>"

// synthesis syn_edif_scalar_format = "%u"



//  input         FRAMEQ_N;

//  input         TRDYQ_N;

//  input         IRDYQ_N;

//  input         STOPQ_N;

//  input         DEVSELQ_N;

  input  [31:0] ADDR;

  inout  [31:0] ADIO;

//  input         CFG_VLD;

//  input         CFG_HIT;

  output        C_TERM;

  output        C_READY;

//  input         ADDR_VLD;

  input   [7:0] BASE_HIT;

  output        S_TERM;

  output        S_READY;

  output        S_ABORT;

  input         S_WRDN;

  input         S_SRC_EN;

  input         S_DATA_VLD;

  input   [3:0] S_CBE;

//  input  [15:0] PCI_CMD;

  output        REQUEST;

  output        REQUESTHOLD;

  output        COMPLETE;

  output        M_WRDN;

  output        M_READY;

//  input         M_SRC_EN;

//  input         M_DATA_VLD;

  output  [3:0] M_CBE;

//  input         TIME_OUT;

  output        CFG_SELF;

//  input         M_DATA;

//  input         DR_BUS;

//  input         I_IDLE;

//  input         M_ADDR_N;

//  input         IDLE;

//  input         B_BUSY;

  input         S_DATA;

//  input         BACKOFF;

  output        INTR_N;

//  input         PERRQ_N;

//  input         SERRQ_N;

  output        KEEPOUT;

//  input  [39:0] CSR;

  output [31:0] SUB_DATA;

//  input [255:0] CFG;

  input         RST;

  input         CLK;







output [10:0]	SD_ADDR; 	// The SDRAM address

output [1:0]   	SD_BS;		// The SDRAM bank select

//output 			SD_CLK;		// The SDRAM clock

output			SD_CKE;		// The SDRAM clock enable

output 			SD_CS_N;	// The SDRAM chip select

output 			SD_RAS_N;	// The SDRAM row address strobe

output         	SD_CAS_N;	// The SDRAM column address strobe\

output 			SD_WE_N;	// The SDRAM write enable

output [3:0]	SD_DQM;		// The SDRAM data masks

inout [31:0]  	SD_DATA;    // The SDRAM data



   output			DISPLAY1A;

   output         DISPLAY1B;

   output         DISPLAY1C;

   output         DISPLAY1D;

   output         DISPLAY1E;

   output         DISPLAY1F;

   output         DISPLAY1G;

   output         DISPLAY2A;

   output         DISPLAY2B;

   output         DISPLAY2C;

   output         DISPLAY2D;

   output         DISPLAY2E;

   output         DISPLAY2F;

   output     		DISPLAY2G;



   input  [7:0]   DIP;

   input          PUSH_USER1;

   input          SIN;

   output         SOUT;

   input [53:0]   SIG_IN;

   output [53:0]  SIG_OUT;

   output         LED_USER;



wire			write_pop;

wire			write_push;

wire			write_push_pci; 		// The PCI interface is pushing data on the FIFO

wire			write_push_pad;			// The write FIFO is being padded

wire			write_flush;

wire			read_pop;

wire 			read_push;

wire			read_flush;



wire        pre_load;            // Indicates that the read data registers should be loaded



wire [3:0]  byte_enable_in;      // The byte enables being sent to the FIFO



wire [31:0]		sdram_write_data;

wire [31:0]     sdram_read_data;

//wire [31:0]     sdram_data;



wire [31:0]    sd_data_in;

wire [31:0]    sd_data_out;

wire           sd_data_enable;



reg [31:0]     current_read;      	// The current value of data to be read

reg [31:0]     previous_read;    	// The previous value of data to be read



wire [3:0]      sdram_write_byte_enable;

wire [24:0]		address;

wire			ren;

wire			wen;

wire			dvld;

wire			transfer;

wire [31:0] 	read_data;



wire 			write_underway;

wire			read_underway;

wire			pci_read_underway;

wire			pci_write_underway;



wire	[24:0]		read_addr;

wire [24:0]		write_addr;



wire 			pci_read_hit;

wire			pci_write_hit;



wire        register_read_hit;

wire        register_write_hit;



wire		register_read_enable;  // A read is underway from a register

wire [31:0]	register_read_data;    // the data from the register block



reg        load_current;  // Indicates that the read_data registers should be loaded

reg			curr_oe;			// Indicates which data should be read



wire [2:0]		w_read_data_count;

wire [2:0]		w_write_data_count;

wire [2:0]		r_read_data_count;

wire [2:0]		r_write_data_count;



wire           begin_write;

wire           begin_write_ok;

wire           pad_write;



wire           sdram_ready;

wire           sdram_term;

wire           sdram_abort;



wire           register_ready;

wire           register_term;

wire           register_abort;    



wire  [31:0]   uart_ctrl_reg;

wire           uart_wen_txfifo;

wire  [7:0]    uart_wdata_txfifo;

wire           uart_ren_rxfifo;

wire  [7:0]    uart_rdata_rxfifo;

wire           uart_rx_of;

wire  [15:0]    uart_fifo_status;



reg [6:0] out_1, out_2;

wire [3:0] seven_seg_reg1, seven_seg_reg2;





  parameter TDLY = 1;



// Assign the signals for a register hit

assign register_read_hit = (BASE_HIT[0] && ~S_WRDN);

assign register_write_hit = (BASE_HIT[0] && S_WRDN);



// Assign the signsls for a PCI_HIt

assign pci_read_hit = (BASE_HIT[1] && ~S_WRDN);

assign pci_write_hit = (BASE_HIT[1] && S_WRDN);



write_fifo iWRITE_FIFO (

		.CLK (CLK),

		.RESET (RST),

		.PUSH (write_push),

		.POP (write_pop),

		.FLUSH (write_flush),

		.WRITE_DATA_COUNT (w_write_data_count),

		.READ_DATA_COUNT (w_read_data_count),

		.DATA_OUT (sdram_write_data),

		.BYTE_ENABLE_OUT (sdram_write_byte_enable),

		.DATA_IN (ADIO),

		.BYTE_ENABLE_IN (byte_enable_in)

		);



assign write_pop = (write_underway) ? transfer : 1'b0;



assign write_push_pci = (pci_write_underway) ? S_DATA_VLD : 1'b0;

assign write_push_pad = pad_write;

assign write_push = write_push_pci || write_push_pad;



assign byte_enable_in = (pad_write) ? 4'b1111 : S_CBE;



read_fifo iREAD_FIFO (

		.CLK (CLK),

		.RESET (RST),

		.PUSH (read_push),

		.POP (read_pop),

		.FLUSH (read_flush),

		.WRITE_DATA_COUNT (r_write_data_count),

		.READ_DATA_COUNT (r_read_data_count),

		.DATA_OUT (read_data),

		.DATA_IN (sdram_read_data)

		);



assign #TDLY ADIO = (curr_oe && pci_read_underway) ? current_read : 32'bz;

assign #TDLY ADIO = (!curr_oe && S_DATA && !S_WRDN && pci_read_underway) ? previous_read : 32'bz;

assign #TDLY ADIO = (register_read_enable) ? register_read_data : 32'bz;

//assign sdram_read_data = sdram_data;

assign read_push = (read_underway) ? dvld : 1'b0;

assign read_pop = S_SRC_EN || pre_load;





assign S_TERM = sdram_term || register_term;

assign S_READY = sdram_ready || register_ready;

assign S_ABORT = sdram_abort || register_abort;



// Load current is a delayed version of FIFO_POP

always @(posedge RST or posedge CLK)

   begin

   if (RST)

      begin

      load_current <= 1'b0;

      curr_oe <= 1'b0;

      end

   else

      begin

      load_current <= read_pop;

      curr_oe <= read_pop;

      end

   end



//Load the current and previous registers which are used for initiator wait states during reads

always @(posedge RST or posedge CLK)

   begin

   if (RST)

      begin

      current_read <= 32'h00000000;

      previous_read <= 32'h00000000;

      end

   else

      if (load_current)

         begin

         current_read <= read_data;

         previous_read <= current_read;

         end

   end





SDRAM_CONTROLLER iSDRAM_CTRL (

		// host signals

		.CLK (CLK),

		.RESET (RST),

		.ADDR (address [22:0]),

		.REN 		(ren),

		.WEN 		(wen),

		.DVLD 		(dvld),

		.TRANSFER 	(transfer),

		.READY 		(ready),

		.SDRAM_WRITE_DATA	(sdram_write_data),

      .SDRAM_READ_DATA  (sdram_read_data),

		.BYTE_MASK 	(sdram_write_byte_enable),



		// SDRAM signals

		.SD_ADDR 	(SD_ADDR),

		.SD_BS   	(SD_BS),

	//	.SD_CLK 	(SD_CLK),

		.SD_CKE		(SD_CKE),

		.SD_CS_N	(SD_CS_N),

		.SD_RAS_N	(SD_RAS_N),

		.SD_CAS_N	(SD_CAS_N),

		.SD_WE_N	(SD_WE_N),

		.SD_DQM		(SD_DQM),

		.SD_DATA_IN	(sd_data_in),

      .SD_DATA_OUT (sd_data_out),

      .SD_DATA_ENABLE (sd_data_enable)

	);



assign  sd_data_in = SD_DATA;

assign  SD_DATA = (sd_data_enable) ? sd_data_out : 32'bz;



assign address = (write_underway) ? write_addr : read_addr;



fifo_sdram_interface_controller iFSIC (

		.CLK (CLK),

		.RESET (RST),

      .BEGIN_WRITE (begin_write),

		.BEGIN_WRITE_OK (begin_write_ok),

		.BEGIN_READ (begin_read),

		.READ_UNDERWAY (read_underway),

		.WRITE_UNDERWAY (write_underway),

		.REN (ren),

		.WEN (wen),

		.TRANSFER (transfer),

		.DVLD (dvld),

		.READ_COMPLETE (read_complete),

		.WRITE_COMPLETE (write_complete)

		);



fifo_pci_interface_controller iFPIC(

		.CLK (CLK),

		.RESET (RST),

		.ADDR (ADDR),

		.READ_HIT (pci_read_hit),

		.WRITE_HIT (pci_write_hit),

		.S_DATA (S_DATA),

		.SDRAM_READY (ready),

		.PCI_READ_UNDERWAY (pci_read_underway),

		.PCI_WRITE_UNDERWAY (pci_write_underway),

      .READ_PRE_LOAD (pre_load),

		.WRITE_COMPLETE (write_complete),

		.READ_COMPLETE (read_complete),

		.BEGIN_WRITE (begin_write),

		.BEGIN_READ (begin_read),

		.READ_ADDRESS (read_addr),

		.WRITE_ADDRESS (write_addr),

		.READ_FLUSH (read_flush),

		.WRITE_FLUSH (write_flush),

      .WRITE_COUNT (w_write_data_count),

      .READ_COUNT (r_read_data_count),

		.READY (sdram_ready),

		.TERM (sdram_term),

		.ABORT (sdram_abort)

		);



// If there is not a full write of 8 items, we need to pad the data

WRITE_PADDER iWRITE_PADDER (

   .CLK (CLK),

   .RESET (RST),

   .BEGIN_WRITE (begin_write),

   .BEGIN_WRITE_OK (begin_write_ok),

   .WRITE_COUNT (w_write_data_count),

   .PAD_WRITE (pad_write)

   );







	//**********************************************************************

   // This section contains the register module

   //*********************************************************************



   REGISTER_MODULE iREG_BLOCK(

            .CLK							(CLK),

            .RESET						(RST),



            .REGISTER_WRITE_HIT		(register_write_hit),

            .REGISTER_READ_HIT		(register_read_hit),

            .S_DATA						(S_DATA),

            .ADDRESS						(ADDR[5:2]),



            .REGISTER_WRITE_DATA		(ADIO),

            .REGISTER_READ_ENABLE	(register_read_enable),

            .REGISTER_READ_DATA		(register_read_data),



            .REGISTER_READY			(register_ready),

            .REGISTER_TERM				(register_term),

            .REGISTER_ABORT         (register_abort),

			

            // UART CORE interface

            .UART_CTRL_REG          (uart_ctrl_reg),

            .UART_WEN_TXFIFO        (uart_wen_txfifo),

            .UART_WDATA_TXFIFO      (uart_wdata_txfifo),

            .UART_REN_RXFIFO        (uart_ren_rxfifo),

            .UART_RDATA_RXFIFO      (uart_rdata_rxfifo),

            .UART_RX_OF             (uart_rx_of),

            .UART_FIFO_STATUS       (uart_fifo_status),



//			   .WRITE_PUSH				(write_push),

//            .READ_PUSH				(read_push),

//            .SDRAM_DATA				(sdram_read_data),

            

            .LED_USER            (LED_USER),

            .SEVEN_SEG1          (seven_seg_reg1),

            .SEVEN_SEG2          (seven_seg_reg2),

            .DIP                 (DIP),   

            .PUSH_USER1          (PUSH_USER1),

            .SIG_IN              (SIG_IN),

            .SIG_OUT             (SIG_OUT),

            .INTR_N              (INTR_N)

            );





UART_CORE iUART_CORE(

   .RESET               (RST),

   .CLK                 (CLK),

   .CTRL_REG            (uart_ctrl_reg),

   .WEN_TXFIFO          (uart_wen_txfifo),

   .WDATA_TXFIFO        (uart_wdata_txfifo),

   .REN_RXFIFO          (uart_ren_rxfifo),

   .RDATA_RXFIFO        (uart_rdata_rxfifo),

   .RX_OF               (uart_rx_of),

   .FIFO_STATUS         (uart_fifo_status),

   .SIN                 (SIN),

   .SOUT                (SOUT)

   );

   //

  //******************************************************************//

  // This section contains unused PCI signals.                            //

  //******************************************************************//







  reg           COMPLETE_reg;

  reg           M_WRDN_reg;

  reg           M_READY_reg;

  reg           INTR_N_reg;

  reg [3:0]     M_CBE_reg;

  reg          C_TERM_reg;

  reg          C_READY_reg;

  reg          KEEPOUT_reg;

  reg          CFG_SELF_reg;

  reg          REQUESTHOLD_reg;

  reg [31:0]   SUB_DATA_reg;

  reg          REQUEST_reg;



  always @(posedge CLK or posedge RST)

  begin : dont_optimize_please

    if (RST)

    begin

      COMPLETE_reg <= 1'b0;

      M_WRDN_reg <= 1'b1;

      M_READY_reg <= 1'b0;

      INTR_N_reg <= 1'b0;

      M_CBE_reg <= 4'b1001;

      C_TERM_reg <= 1'b0;

      C_READY_reg <= 1'b0;

      KEEPOUT_reg <= 1'b1;

      CFG_SELF_reg <= 1'b1;

      REQUESTHOLD_reg <= 1'b0;

      SUB_DATA_reg <= 32'hFFFFFFFF;

      REQUEST_reg <= 1'b0;

    end

    else

    begin

      COMPLETE_reg <= 1'b1;

      M_WRDN_reg <= 1'b0;

      M_READY_reg <= 1'b1;

      INTR_N_reg <= 1'b1;

      M_CBE_reg <= 4'b0110;

      C_TERM_reg <= 1'b1;

      C_READY_reg <= 1'b1;

      KEEPOUT_reg <= 1'b0;

      CFG_SELF_reg <= 1'b0;

      REQUESTHOLD_reg <= 1'b0;

      SUB_DATA_reg <= 32'h00000000;

      REQUEST_reg <= 1'b0;

    end

  end



  //assign #TDLY INTR_N = INTR_N_reg;

  assign #TDLY COMPLETE = COMPLETE_reg;

  assign #TDLY M_WRDN = M_WRDN_reg;

  assign #TDLY M_CBE = M_CBE_reg;

  assign #TDLY M_READY = M_READY_reg;



  assign #TDLY C_TERM = C_TERM_reg;

  assign #TDLY C_READY = C_READY_reg;



  assign #TDLY KEEPOUT = KEEPOUT_reg;

  assign #TDLY CFG_SELF = CFG_SELF_reg;

  assign #TDLY REQUESTHOLD = REQUESTHOLD_reg;

  assign #TDLY SUB_DATA = SUB_DATA_reg;

  assign #TDLY REQUEST = REQUEST_reg;



  // A seven segment display module added to help with debugging the board

   always @ (posedge CLK or posedge RST)

   begin

      if (RST)

         out_1 <= 7'h0;

      else

         case (seven_seg_reg1)

            4'h0 : out_1 <= 7'h7E;

            4'h1 : out_1 <= 7'h30;

            4'h2 : out_1 <= 7'h6d;

            4'h3 : out_1 <= 7'h79;

            4'h4 : out_1 <= 7'h33;

            4'h5 : out_1 <= 7'h5B;

            4'h6 : out_1 <= 7'h5F;

            4'h7 : out_1 <= 7'h70;

            4'h8 : out_1 <= 7'h7F;

            4'h9 : out_1 <= 7'h7B;

            4'hA : out_1 <= 7'h77;

            4'hB : out_1 <= 7'h1F;

            4'hC : out_1 <= 7'h4E;

            4'hD : out_1 <= 7'h3D;

            4'hE : out_1 <= 7'h4F;

            4'hF : out_1 <= 7'h47;

            default : out_1 <= 7'h00;

         endcase

   end



   assign DISPLAY1A = out_1[6];

   assign DISPLAY1B = out_1[5];

   assign DISPLAY1C = out_1[4];

   assign DISPLAY1D = out_1[3];

   assign DISPLAY1E = out_1[2];

   assign DISPLAY1F = out_1[1];

   assign DISPLAY1G = out_1[0];



   always @ (posedge CLK or posedge RST)

   begin

      if (RST)

         out_2 <= 7'h0;

      else

         case (seven_seg_reg2)

            4'h0 : out_2 <= 7'h7E;

            4'h1 : out_2 <= 7'h30;

            4'h2 : out_2 <= 7'h6d;

            4'h3 : out_2 <= 7'h79;

            4'h4 : out_2 <= 7'h33;

            4'h5 : out_2 <= 7'h5B;

            4'h6 : out_2 <= 7'h5F;

            4'h7 : out_2 <= 7'h70;

            4'h8 : out_2 <= 7'h7F;

            4'h9 : out_2 <= 7'h7B;

            4'hA : out_2 <= 7'h77;

            4'hB : out_2 <= 7'h1F;

            4'hC : out_2 <= 7'h4E;

            4'hD : out_2 <= 7'h3D;

            4'hE : out_2 <= 7'h4F;

            4'hF : out_2 <= 7'h47;

            default : out_2 <= 7'h00;

         endcase

   end



   assign DISPLAY2A = out_2[6];

   assign DISPLAY2B = out_2[5];

   assign DISPLAY2C = out_2[4];

   assign DISPLAY2D = out_2[3];

   assign DISPLAY2E = out_2[2];

   assign DISPLAY2F = out_2[1];

   assign DISPLAY2G = out_2[0];



endmodule