#include "driver.h"



#ifdef ALLOC_PRAGMA

#pragma alloc_text (INIT, DriverEntry)

#pragma alloc_text (PAGE, Unload)

#pragma alloc_text (PAGE, Dispatch)

#endif



// IRQL PASSIVE_LEVEL

NTSTATUS

DriverEntry (

  IN PDRIVER_OBJECT     DriverObject,

  IN PUNICODE_STRING    RegistryPath

  )

{

  NTSTATUS  status = STATUS_SUCCESS;



  KdPrint (("ledctrl.sys: DriverEntry\n"));



  // initialize the driver dispatch table

  DriverObject->DriverExtension->AddDevice    = AddDevice;

  DriverObject->DriverUnload                  = Unload;

  DriverObject->MajorFunction[IRP_MJ_PNP]     = DispatchPnP;

  DriverObject->MajorFunction[IRP_MJ_POWER]   = DispatchPower;

  DriverObject->MajorFunction[IRP_MJ_CREATE]  = Dispatch;

  DriverObject->MajorFunction[IRP_MJ_CLOSE]   = Dispatch;

  DriverObject->MajorFunction[IRP_MJ_READ]    = Dispatch;

  DriverObject->MajorFunction[IRP_MJ_WRITE]   = Dispatch;

  DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL]  = Dispatch;

  // initialize driver variables



  // read or set driver values in the registry



  // save the RegistryPath



  KdPrint (("ledctrl.sys: DriverEntry complete\n"));



  return status;

} // DriverEntry



// IRQL PASSIVE_LEVEL

VOID

Unload (

  IN PDRIVER_OBJECT     DriverObject

  )

{

  PAGED_CODE ();



  KdPrint (("ledctrl.sys: DriverUnload\n"));



  return;

} // Unload





// IRQL PASSIVE_LEVEL

NTSTATUS

Dispatch (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  )

{

  NTSTATUS              status = STATUS_SUCCESS;

  PDEVICE_EXTENSION     deviceExtension;

  PIO_STACK_LOCATION    ioStack;

  ULONG                 information;

  PULONG                pBuffer;

  ULONG                 length;

  ULONG                 address;





  PAGED_CODE ();



  KdPrint (("ledctrl.sys: Dispatch\n"));



  deviceExtension = (PDEVICE_EXTENSION) DeviceObject->DeviceExtension;

  ioStack = IoGetCurrentIrpStackLocation (Irp);



  switch (ioStack->MajorFunction)  {

  case IRP_MJ_CREATE:

    KdPrint (("ledctrl.sys: Create\n"));

    information = 0;

    break;



  case IRP_MJ_CLOSE:

    KdPrint (("ledctrl.sys: Close\n"));

    information = 0;

    break;



  case IRP_MJ_READ:

    KdPrint (("ledctrl.sys: Read\n"));



    pBuffer = (PULONG) Irp->AssociatedIrp.SystemBuffer;

    length = ioStack->Parameters.Read.Length / sizeof (ULONG);

    address = (ULONG) deviceExtension->MemoryStart + deviceExtension->MemoryOffset;

    KdPrint (("  read from: 0x%x, to: 0x%x", pBuffer, address));

    READ_REGISTER_BUFFER_ULONG ((PULONG) address, pBuffer, length);

    information = length * sizeof (ULONG);

    break;



  case IRP_MJ_WRITE:

    KdPrint (("ledctrl.sys: Write\n"));



    pBuffer = (PULONG) Irp->AssociatedIrp.SystemBuffer;

    length = ioStack->Parameters.Write.Length / sizeof (ULONG);

    address = (ULONG) deviceExtension->MemoryStart + deviceExtension->MemoryOffset;

    KdPrint (("  write from: 0x%x, to: 0x%x", address, pBuffer));

    WRITE_REGISTER_BUFFER_ULONG ((PULONG) address, pBuffer, length);

    information = length * sizeof (ULONG);

    break;



  case IRP_MJ_DEVICE_CONTROL:

    KdPrint (("ledctrl.sys: DeviceControl\n"));

    switch (ioStack->Parameters.DeviceIoControl.IoControlCode)  {

    case IO_CTL_SET_OFFSET:

      pBuffer = (PULONG) Irp->AssociatedIrp.SystemBuffer;

      RtlCopyMemory (&deviceExtension->MemoryOffset, pBuffer, sizeof (ULONG));

      KdPrint (("  offset is 0x%x", deviceExtension->MemoryOffset));

      break;

    }



    break;

  }



  Irp->IoStatus.Status = status;

  Irp->IoStatus.Information = information;

  IoCompleteRequest (Irp, IO_NO_INCREMENT);



  KdPrint (("ledctrl.sys: Dispatch complete\n"));



  return status;

} // Dispatch



