#ifndef _DRIVER_H_

#define _DRIVER_H_



#include <wdm.h>



// defined types

typedef enum _DEVICE_PNP_STATE {



  NotStarted = 0,         // Not started yet

  Deleted,                // Device has received the REMOVE_DEVICE IRP

  SurpriseRemovePending,  // Device has received the SURPRISE_REMOVE IRP

  Stopped,                // Device has received the STOP_DEVICE IRP

  RemovePending,          // Device has received the QUERY_REMOVE IRP

  FailHardware,           // Fail hardware transfers

  StopPending,            // Device has received the QUERY_STOP IRP

  FailSoftware,           // Fail incoming IRPs

  Started                 // Device has received the START_DEVICE IRP



} DEVICE_PNP_STATE;



#define IO_CTL_SET_OFFSET  CTL_CODE ( \

  FILE_DEVICE_UNKNOWN,                \

  0x000,                              \

  METHOD_BUFFERED,                    \

  FILE_ANY_ACCESS)



typedef struct _DEVICE_EXTENSION  {



  PDEVICE_OBJECT        PhysicalDeviceObject;

  PDEVICE_OBJECT        FunctionDeviceObject;

  PDEVICE_OBJECT        NextStackDeviceObject;



  UNICODE_STRING        InterfaceName;



  DEVICE_PNP_STATE      DevicePnPState;

  DEVICE_PNP_STATE      PreviousPnPState;



  PHYSICAL_ADDRESS      Memory;

  PVOID                 MemoryStart;

  ULONG                 MemoryOffset;

  ULONG                 MemoryLength;



} DEVICE_EXTENSION, *PDEVICE_EXTENSION;



// macros

#define INITIALIZE_PNP_STATE(_Data_)    \

        (_Data_)->DevicePnPState =  NotStarted;\

        (_Data_)->PreviousPnPState = NotStarted;



#define SET_NEW_PNP_STATE(_Data_, _state_) \

        (_Data_)->PreviousPnPState =  (_Data_)->DevicePnPState;\

        (_Data_)->DevicePnPState = (_state_);



#define RESTORE_PREVIOUS_PNP_STATE(_Data_)   \

        (_Data_)->DevicePnPState =   (_Data_)->PreviousPnPState;



// forward declarations



// driver.c

NTSTATUS

DriverEntry (

  IN PDRIVER_OBJECT     DriverObject,

  IN PUNICODE_STRING    RegistryPath

  );



VOID

Unload (

  IN PDRIVER_OBJECT     DriverObject

  );



NTSTATUS

Dispatch (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



// pnp.c

NTSTATUS

AddDevice (

  IN PDRIVER_OBJECT     DriverObject,

  IN PDEVICE_OBJECT     PhysicalDeviceObject

  );



NTSTATUS

DispatchPnP (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



NTSTATUS

DispatchPower (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



#endif // _DRIVER_H_



#ifdef GUID_DRIVER



// {51F83ACE-270D-4bad-B3A9-8957D84FD252}

static const GUID GuidDriver = 

{ 0x51f83ace, 0x270d, 0x4bad, { 0xb3, 0xa9, 0x89, 0x57, 0xd8, 0x4f, 0xd2, 0x52 } };



#endif // GUID_DRIVER



